'''OpenGL extension ARB.texture_query_levels

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_query_levels to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a new set of texture functions
	(textureQueryLevels) in the OpenGL Shading Language that exposes the
	number of accessible mipmap levels in the texture associated with a GLSL
	sampler variable.  The set of accessible levels includes all the levels of
	the texture defined either through TexImage*, TexStorage*, or TextureView*
	(ARB_texture_view) APIs that are not below the TEXTURE_BASE_LEVEL or above
	the TEXTURE_MAX_LEVEL parameters.  For textures defined with TexImage*,
	the set of resident levels is somewhat implementation-dependent.  For
	fully defined results, applications should use TexStorage*/TextureView
	unless the texture has a full mipmap chain and is used with a mipmapped
	minification filter.
	
	These functions means that shaders are not required to manually recompute,
	approximate, or maintain a uniform holding a pre-computed level count,
	since the true level count is already available to the
	implementation. This value can be used to avoid black or leaking pixel
	artifacts for rendering methods which are using texture images as memory
	pages (eg: virtual textures); methods that can't only rely on the fixed
	pipeline texture functions which take advantage of TEXTURE_MAX_LEVEL for
	their sampling.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_query_levels.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_query_levels import *
from OpenGL.raw.GL.ARB.texture_query_levels import _EXTENSION_NAME

def glInitTextureQueryLevelsARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.vdpau_interop2

This module customises the behaviour of the 
OpenGL.raw.GL.NV.vdpau_interop2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows VDPAU video surfaces to be used
	either with frame or field structures for texturing and rendering.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/vdpau_interop2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.vdpau_interop2 import *
from OpenGL.raw.GL.NV.vdpau_interop2 import _EXTENSION_NAME

def glInitVdpauInterop2NV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glVDPAURegisterVideoSurfaceWithPictureStructureNV.textureNames size not checked against numTextureNames
glVDPAURegisterVideoSurfaceWithPictureStructureNV=wrapper.wrapper(glVDPAURegisterVideoSurfaceWithPictureStructureNV).setInputArraySize(
    'textureNames', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLX import _types as _cs
# End users want this...
from OpenGL.raw.GLX._types import *
from OpenGL.raw.GLX import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLX_SGIX_pbuffer'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLX,'GLX_SGIX_pbuffer',error_checker=_errors._error_checker)
GLX_ACCUM_BUFFER_BIT_SGIX=_C('GLX_ACCUM_BUFFER_BIT_SGIX',0x00000080)
GLX_AUX_BUFFERS_BIT_SGIX=_C('GLX_AUX_BUFFERS_BIT_SGIX',0x00000010)
GLX_BACK_LEFT_BUFFER_BIT_SGIX=_C('GLX_BACK_LEFT_BUFFER_BIT_SGIX',0x00000004)
GLX_BACK_RIGHT_BUFFER_BIT_SGIX=_C('GLX_BACK_RIGHT_BUFFER_BIT_SGIX',0x00000008)
GLX_BUFFER_CLOBBER_MASK_SGIX=_C('GLX_BUFFER_CLOBBER_MASK_SGIX',0x08000000)
GLX_DAMAGED_SGIX=_C('GLX_DAMAGED_SGIX',0x8020)
GLX_DEPTH_BUFFER_BIT_SGIX=_C('GLX_DEPTH_BUFFER_BIT_SGIX',0x00000020)
GLX_EVENT_MASK_SGIX=_C('GLX_EVENT_MASK_SGIX',0x801F)
GLX_FRONT_LEFT_BUFFER_BIT_SGIX=_C('GLX_FRONT_LEFT_BUFFER_BIT_SGIX',0x00000001)
GLX_FRONT_RIGHT_BUFFER_BIT_SGIX=_C('GLX_FRONT_RIGHT_BUFFER_BIT_SGIX',0x00000002)
GLX_HEIGHT_SGIX=_C('GLX_HEIGHT_SGIX',0x801E)
GLX_LARGEST_PBUFFER_SGIX=_C('GLX_LARGEST_PBUFFER_SGIX',0x801C)
GLX_MAX_PBUFFER_HEIGHT_SGIX=_C('GLX_MAX_PBUFFER_HEIGHT_SGIX',0x8017)
GLX_MAX_PBUFFER_PIXELS_SGIX=_C('GLX_MAX_PBUFFER_PIXELS_SGIX',0x8018)
GLX_MAX_PBUFFER_WIDTH_SGIX=_C('GLX_MAX_PBUFFER_WIDTH_SGIX',0x8016)
GLX_OPTIMAL_PBUFFER_HEIGHT_SGIX=_C('GLX_OPTIMAL_PBUFFER_HEIGHT_SGIX',0x801A)
GLX_OPTIMAL_PBUFFER_WIDTH_SGIX=_C('GLX_OPTIMAL_PBUFFER_WIDTH_SGIX',0x8019)
GLX_PBUFFER_BIT_SGIX=_C('GLX_PBUFFER_BIT_SGIX',0x00000004)
GLX_PBUFFER_SGIX=_C('GLX_PBUFFER_SGIX',0x8023)
GLX_PRESERVED_CONTENTS_SGIX=_C('GLX_PRESERVED_CONTENTS_SGIX',0x801B)
GLX_SAMPLE_BUFFERS_BIT_SGIX=_C('GLX_SAMPLE_BUFFERS_BIT_SGIX',0x00000100)
GLX_SAVED_SGIX=_C('GLX_SAVED_SGIX',0x8021)
GLX_STENCIL_BUFFER_BIT_SGIX=_C('GLX_STENCIL_BUFFER_BIT_SGIX',0x00000040)
GLX_WIDTH_SGIX=_C('GLX_WIDTH_SGIX',0x801D)
GLX_WINDOW_SGIX=_C('GLX_WINDOW_SGIX',0x8022)
@_f
@_p.types(_cs.GLXPbufferSGIX,ctypes.POINTER(_cs.Display),_cs.GLXFBConfigSGIX,_cs.c_uint,_cs.c_uint,ctypes.POINTER(_cs.c_int))
def glXCreateGLXPbufferSGIX(dpy,config,width,height,attrib_list):pass
@_f
@_p.types(None,ctypes.POINTER(_cs.Display),_cs.GLXPbufferSGIX)
def glXDestroyGLXPbufferSGIX(dpy,pbuf):pass
@_f
@_p.types(None,ctypes.POINTER(_cs.Display),_cs.GLXDrawable,ctypes.POINTER(_cs.c_ulong))
def glXGetSelectedEventSGIX(dpy,drawable,mask):pass
@_f
@_p.types(None,ctypes.POINTER(_cs.Display),_cs.GLXPbufferSGIX,_cs.c_int,ctypes.POINTER(_cs.c_uint))
def glXQueryGLXPbufferSGIX(dpy,pbuf,attribute,value):pass
@_f
@_p.types(None,ctypes.POINTER(_cs.Display),_cs.GLXDrawable,_cs.c_ulong)
def glXSelectEventSGIX(dpy,drawable,mask):pass



"""Utility module for interacting with GBM to select rendering device

The base code here comes from github bug report #6 in the pyopengl
project, thanks to @abandoned-cocoon for that.
"""
import weakref, ctypes, logging, os, glob
from OpenGL.platform import ctypesloader
from OpenGL import _opaque
log = logging.getLogger(__name__)
gbm = ctypesloader.loadLibrary(ctypes.CDLL,"gbm")
__all__ = ('enumerate_devices','open_device','close_device','gbm')
_DEVICE_HANDLES = {}
GBM_BO_USE_SCANOUT = (1 << 0)
GBM_BO_USE_CURSOR = (1 << 1)
GBM_BO_USE_CURSOR_64X64 = GBM_BO_USE_CURSOR
GBM_BO_USE_RENDERING = (1 << 2)
GBM_BO_USE_WRITE = (1 << 3)
GBM_BO_USE_LINEAR = (1 << 4)

GBMDevice = _opaque.opaque_pointer_cls('GBMDevice')
GBMSurface = _opaque.opaque_pointer_cls('GBMSurface')
gbm.gbm_create_device.restype = GBMDevice
gbm.gbm_surface_create.restype = GBMSurface

def filter_bad_drivers(cards, bad_drivers=('nvidia',)):
    """Lookup the driver for each card to exclude loading nvidia devices"""
    # this is pci specific, which I suppose means we're going to fail
    # if the GPU isn't on the PCI bus, but we don't seem to have
    # another way to match up the card to the driver :(
    bad_cards = set()
    for link in glob.glob('/dev/dri/by-path/pci-*-card'):
        base = os.path.basename(link)
        pci_id = base[4:-5]
        driver = os.path.basename(os.readlink('/sys/bus/pci/devices/%s/driver'%(pci_id,)))
        if driver in bad_drivers:
            card = os.path.basename(os.readlink(link))
            log.debug("Skipping %s because it uses %s driver",card,driver)
            bad_cards.add(card)
    filtered = [
        card for card in cards 
        if os.path.basename(card) not in bad_cards
    ]
    # assert len(filtered) == 1, filtered
    return filtered

def enumerate_devices():
    """Enumerate the set of gbm renderD* devices on the system
    
    Attempts to filter out any nvidia drivers with filter_bad_drivers
    along the way.
    """
    import glob
    # gbm says card* is the correct entry point...
    return filter_bad_drivers(sorted(glob.glob('/dev/dri/card*')))
def open_device(path):
    """Open a particular gbm device
    
    * path -- integer index of devices in sorted enumeration, or
              device basename (`renderD128`) or a full path-name
              as returned from enumerate_devices

    Will raise (at least):

    * RuntimeError for invalid indices
    * IOError/OSError for device access failures
    * RuntimeError if we cannot create the gbm device

    Caller is responsible for calling close_device(display) on the 
    resulting opaque pointer in order to release the open file-handle
    and deallocate the gbm_device record.

    returns GBMDevice, an opaque pointer
    """
    if isinstance(path,int):
        try:
            devices = enumerate_devices()
            path = devices[int]
        except IndexError:
            raise RuntimeError('Only %s devices available, cannot use 0-index %s'%(len(devices),path))
    else:
        path = os.path.join('/dev/dri',path) # allow for specifying "renderD128"
    log.debug("Final device path: %s", path)
    fh = open(path,'w')
    dev = gbm.gbm_create_device(fh.fileno())
    if dev == 0:
        fh.close()
        raise RuntimeError('Unable to create rendering device for: %s'%(path))
    _DEVICE_HANDLES[dev] = fh
    return dev
def close_device(device):
    """Close an opened gbm device"""
    gbm.gbm_device_destroy(device)
    try:
        handle = _DEVICE_HANDLES.pop(device)
        handle.close()
    except KeyError:
        pass

def create_surface(device, width=512, height=512, format=None, flags=GBM_BO_USE_RENDERING):
    """Create a GBM surface to use on the given device
    
    devices -- opaque GBMDevice pointer
    width,height -- dimensions
    format -- EGL_NATIVE_VISUAL_ID from an EGL configuration
    flags -- surface flags regarding reading/writing pattern that
             is expected for the buffer
    
    returns GBMSurface opaque pointer
    """
    if not format:
        raise ValueError(
            'Use eglGetConfigAttrib(display,config,EGL_NATIVE_VISUAL_ID) to get the native surface format',
        )
    return gbm.gbm_surface_create(device, width, height, format, flags)


'''OpenGL extension NV.geometry_shader_passthrough

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.geometry_shader_passthrough to provide a more 
Python-friendly API

Overview (from the spec)
	
	Geometry shaders provide the ability for applications to process each
	primitive sent through the GL using a programmable shader.  While geometry
	shaders can be used to perform a number of different operations, including
	subdividing primitives and changing primitive type, one common use case
	treats geometry shaders as largely "passthrough".  In this use case, the
	bulk of the geometry shader code simply copies inputs from each vertex of
	the input primitive to corresponding outputs in the vertices of the output
	primitive.  Such shaders might also compute values for additional built-in
	or user-defined per-primitive attributes (e.g., gl_Layer) to be assigned
	to all the vertices of the output primitive.
	
	This extension provides a shading language abstraction to express such
	shaders without requiring explicit logic to manually copy attributes from
	input vertices to output vertices.  For example, consider the following
	simple geometry shader in unextended OpenGL:
	
	  layout(triangles) in;
	  layout(triangle_strip) out;
	  layout(max_vertices=3) out;
	
	  in Inputs {
	    vec2 texcoord;
	    vec4 baseColor;
	  } v_in[];
	  out Outputs {
	    vec2 texcoord;
	    vec4 baseColor;
	  };
	
	  void main()
	  {
	    int layer = compute_layer();
	    for (int i = 0; i < 3; i++) {
	      gl_Position = gl_in[i].gl_Position;
	      texcoord = v_in[i].texcoord;
	      baseColor = v_in[i].baseColor;
	      gl_Layer = layer;
	      EmitVertex();
	    }
	  }
	
	In this shader, the inputs "gl_Position", "Inputs.texcoord", and
	"Inputs.baseColor" are simply copied from the input vertex to the
	corresponding output vertex.  The only "interesting" work done by the
	geometry shader is computing and emitting a gl_Layer value for the
	primitive.
	
	The following geometry shader, using this extension, is equivalent:
	
	  #extension GL_NV_geometry_shader_passthrough : require
	
	  layout(triangles) in;
	  // No output primitive layout qualifiers required.
	
	  // Redeclare gl_PerVertex to pass through "gl_Position".
	  layout(passthrough) in gl_PerVertex {
	    vec4 gl_Position;
	  } gl_in[];
	
	  // Declare "Inputs" with "passthrough" to automatically copy members.
	  layout(passthrough) in Inputs {
	    vec2 texcoord;
	    vec4 baseColor;
	  } v_in[];
	
	  // No output block declaration required.
	
	  void main()
	  {
	    // The shader simply computes and writes gl_Layer.  We don't
	    // loop over three vertices or call EmitVertex().
	    gl_Layer = compute_layer();
	  }

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/geometry_shader_passthrough.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.geometry_shader_passthrough import *
from OpenGL.raw.GLES2.NV.geometry_shader_passthrough import _EXTENSION_NAME

def glInitGeometryShaderPassthroughNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_OES_point_size_array'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_OES_point_size_array',error_checker=_errors._error_checker)
GL_POINT_SIZE_ARRAY_BUFFER_BINDING_OES=_C('GL_POINT_SIZE_ARRAY_BUFFER_BINDING_OES',0x8B9F)
GL_POINT_SIZE_ARRAY_OES=_C('GL_POINT_SIZE_ARRAY_OES',0x8B9C)
GL_POINT_SIZE_ARRAY_POINTER_OES=_C('GL_POINT_SIZE_ARRAY_POINTER_OES',0x898C)
GL_POINT_SIZE_ARRAY_STRIDE_OES=_C('GL_POINT_SIZE_ARRAY_STRIDE_OES',0x898B)
GL_POINT_SIZE_ARRAY_TYPE_OES=_C('GL_POINT_SIZE_ARRAY_TYPE_OES',0x898A)
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glPointSizePointerOES(type,stride,pointer):pass



'''OpenGL extension NV.stereo_view_rendering

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.stereo_view_rendering to provide a more 
Python-friendly API

Overview (from the spec)
	
	Virtual reality (VR) applications often render a single logical scene
	from multiple views corresponding to a pair of eyes.  The views (eyes) are
	separated by a fixed offset in the X direction.
	
	Traditionally, multiple views are rendered via multiple rendering passes.
	This is expensive for the GPU because the objects in the scene must be
	transformed, rasterized, shaded, and fragment processed redundantly.  This
	is expensive for the CPU because the scene graph needs to be visited
	multiple times and driver validation happens for each view.  Rendering N
	passes tends to take N times longer than a single pass.
	
	This extension provides a mechanism to render binocular (stereo) views
	from a single stream of OpenGL rendering commands.  Vertex, tessellation,
	and geometry (VTG) shaders can output two positions for each vertex
	corresponding to the two eye views.  A built-in "gl_SecondaryPositionNV"
	is added to specify the second position.  The positions from each view may
	be sent to different viewports and/or layers.  A built-in
	"gl_SecondaryViewportMaskNV[]" is also added to specify the viewport mask
	for the second view.  A new layout-qualifier "secondary_view_offset" is
	added for built-in output "gl_Layer" which allows for the geometry from
	each view to be sent to different layers for rendering.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/stereo_view_rendering.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.stereo_view_rendering import *
from OpenGL.raw.GLES2.NV.stereo_view_rendering import _EXTENSION_NAME

def glInitStereoViewRenderingNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_vertex_attrib_64bit'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_vertex_attrib_64bit',error_checker=_errors._error_checker)
GL_DOUBLE=_C('GL_DOUBLE',0x140A)
GL_DOUBLE_MAT2_EXT=_C('GL_DOUBLE_MAT2_EXT',0x8F46)
GL_DOUBLE_MAT2x3_EXT=_C('GL_DOUBLE_MAT2x3_EXT',0x8F49)
GL_DOUBLE_MAT2x4_EXT=_C('GL_DOUBLE_MAT2x4_EXT',0x8F4A)
GL_DOUBLE_MAT3_EXT=_C('GL_DOUBLE_MAT3_EXT',0x8F47)
GL_DOUBLE_MAT3x2_EXT=_C('GL_DOUBLE_MAT3x2_EXT',0x8F4B)
GL_DOUBLE_MAT3x4_EXT=_C('GL_DOUBLE_MAT3x4_EXT',0x8F4C)
GL_DOUBLE_MAT4_EXT=_C('GL_DOUBLE_MAT4_EXT',0x8F48)
GL_DOUBLE_MAT4x2_EXT=_C('GL_DOUBLE_MAT4x2_EXT',0x8F4D)
GL_DOUBLE_MAT4x3_EXT=_C('GL_DOUBLE_MAT4x3_EXT',0x8F4E)
GL_DOUBLE_VEC2_EXT=_C('GL_DOUBLE_VEC2_EXT',0x8FFC)
GL_DOUBLE_VEC3_EXT=_C('GL_DOUBLE_VEC3_EXT',0x8FFD)
GL_DOUBLE_VEC4_EXT=_C('GL_DOUBLE_VEC4_EXT',0x8FFE)
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLdoubleArray)
def glGetVertexAttribLdvEXT(index,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble)
def glVertexAttribL1dEXT(index,x):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttribL1dvEXT(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble)
def glVertexAttribL2dEXT(index,x,y):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttribL2dvEXT(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glVertexAttribL3dEXT(index,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttribL3dvEXT(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glVertexAttribL4dEXT(index,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,arrays.GLdoubleArray)
def glVertexAttribL4dvEXT(index,v):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glVertexAttribLPointerEXT(index,size,type,stride,pointer):pass



'''OpenGL extension OES.matrix_palette

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.matrix_palette to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds the ability to support vertex skinning in OpenGL ES.
	A simplified version of the ARB_matrix_palette extension is used to
	define OES_matrix_palette extension.
	
	This extension allow OpenGL ES to support a palette of matrices.  The matrix
	palette defines a set of matrices that can be used to transform a vertex.
	The matrix palette is not part of the model view matrix stack and is enabled
	by setting the MATRIX_MODE to MATRIX_PALETTE_OES.
	
	The n vertex units use a palette of m modelview matrices (where n and m are
	constrained to implementation defined maxima.)  Each vertex has a set of n
	indices into the palette, and a corresponding set of n weights.
	Matrix indices and weights can be changed for each vertex.  
	
	When this extension is utilized, the enabled units transform each
	vertex by the modelview matrices specified by the vertices'
	respective indices.  These results are subsequently scaled by the
	weights of the respective units and then summed to create the
	eyespace vertex.   
	
	A similar procedure is followed for normals.  Normals, however,
	are transformed by the inverse transpose of the modelview matrix.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/matrix_palette.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.matrix_palette import *
from OpenGL.raw.GLES1.OES.matrix_palette import _EXTENSION_NAME

def glInitMatrixPaletteOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glMatrixIndexPointerOES.pointer size not checked against 'size,type,stride'
glMatrixIndexPointerOES=wrapper.wrapper(glMatrixIndexPointerOES).setInputArraySize(
    'pointer', None
)
# INPUT glWeightPointerOES.pointer size not checked against 'type,stride'
glWeightPointerOES=wrapper.wrapper(glWeightPointerOES).setInputArraySize(
    'pointer', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.semaphore

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.semaphore to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/semaphore.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.semaphore import *
from OpenGL.raw.GLES2.EXT.semaphore import _EXTENSION_NAME

def glInitSemaphoreEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGetUnsignedBytevEXT.data size not checked against 'pname'
glGetUnsignedBytevEXT=wrapper.wrapper(glGetUnsignedBytevEXT).setInputArraySize(
    'data', None
)
# INPUT glGetUnsignedBytei_vEXT.data size not checked against 'target'
glGetUnsignedBytei_vEXT=wrapper.wrapper(glGetUnsignedBytei_vEXT).setInputArraySize(
    'data', None
)
# INPUT glGenSemaphoresEXT.semaphores size not checked against n
glGenSemaphoresEXT=wrapper.wrapper(glGenSemaphoresEXT).setInputArraySize(
    'semaphores', None
)
# INPUT glDeleteSemaphoresEXT.semaphores size not checked against n
glDeleteSemaphoresEXT=wrapper.wrapper(glDeleteSemaphoresEXT).setInputArraySize(
    'semaphores', None
)
# INPUT glWaitSemaphoreEXT.buffers size not checked against 'numBufferBarriers'
# INPUT glWaitSemaphoreEXT.srcLayouts size not checked against 'numTextureBarriers'
# INPUT glWaitSemaphoreEXT.textures size not checked against 'numTextureBarriers'
glWaitSemaphoreEXT=wrapper.wrapper(glWaitSemaphoreEXT).setInputArraySize(
    'buffers', None
).setInputArraySize(
    'srcLayouts', None
).setInputArraySize(
    'textures', None
)
# INPUT glSignalSemaphoreEXT.buffers size not checked against 'numBufferBarriers'
# INPUT glSignalSemaphoreEXT.dstLayouts size not checked against 'numTextureBarriers'
# INPUT glSignalSemaphoreEXT.textures size not checked against 'numTextureBarriers'
glSignalSemaphoreEXT=wrapper.wrapper(glSignalSemaphoreEXT).setInputArraySize(
    'buffers', None
).setInputArraySize(
    'dstLayouts', None
).setInputArraySize(
    'textures', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension OES.vertex_type_10_10_10_2

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.vertex_type_10_10_10_2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds the following data formats:
	
	Two new vertex attribute data formats: a signed 10.10.10.2 and an unsigned 
	10.10.10.2 vertex data format.  These vertex data formats describe a 3- or 4-tuple 
	stream which can be used to store normals or other attributes in a quantized
	form.  Normals, tangents, binormals and other vertex attributes can often be specified 
	at reduced precision without introducing noticeable artifacts, reducing the 
	amount of memory and memory bandwidth they consume.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/vertex_type_10_10_10_2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.vertex_type_10_10_10_2 import *
from OpenGL.raw.GLES2.OES.vertex_type_10_10_10_2 import _EXTENSION_NAME

def glInitVertexType1010102OES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.vertex_array_object

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.vertex_array_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces vertex array objects which encapsulate
	vertex array states on the server side (vertex buffer objects).
	These objects aim to keep pointers to vertex data and to provide
	names for different sets of vertex data. Therefore applications are
	allowed to rapidly switch between different sets of vertex array
	state, and to easily return to the default vertex array state.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/vertex_array_object.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.vertex_array_object import *
from OpenGL.raw.GLES2.OES.vertex_array_object import _EXTENSION_NAME

def glInitVertexArrayObjectOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDeleteVertexArraysOES.arrays size not checked against n
glDeleteVertexArraysOES=wrapper.wrapper(glDeleteVertexArraysOES).setInputArraySize(
    'arrays', None
)
# INPUT glGenVertexArraysOES.arrays size not checked against n
glGenVertexArraysOES=wrapper.wrapper(glGenVertexArraysOES).setInputArraySize(
    'arrays', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension VERSION.GL_3_3

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_3_3 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_3_3.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.VERSION.GL_3_3 import *
from OpenGL.raw.GL.VERSION.GL_3_3 import _EXTENSION_NAME

def glInitGl33VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGenSamplers=wrapper.wrapper(glGenSamplers).setOutput(
    'samplers',size=lambda x:(x,),pnameArg='count',orPassIn=True
)
# INPUT glDeleteSamplers.samplers size not checked against count
glDeleteSamplers=wrapper.wrapper(glDeleteSamplers).setInputArraySize(
    'samplers', None
)
# INPUT glSamplerParameteriv.param size not checked against 'pname'
glSamplerParameteriv=wrapper.wrapper(glSamplerParameteriv).setInputArraySize(
    'param', None
)
# INPUT glSamplerParameterfv.param size not checked against 'pname'
glSamplerParameterfv=wrapper.wrapper(glSamplerParameterfv).setInputArraySize(
    'param', None
)
# INPUT glSamplerParameterIiv.param size not checked against 'pname'
glSamplerParameterIiv=wrapper.wrapper(glSamplerParameterIiv).setInputArraySize(
    'param', None
)
# INPUT glSamplerParameterIuiv.param size not checked against 'pname'
glSamplerParameterIuiv=wrapper.wrapper(glSamplerParameterIuiv).setInputArraySize(
    'param', None
)
glGetSamplerParameteriv=wrapper.wrapper(glGetSamplerParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetSamplerParameterIiv=wrapper.wrapper(glGetSamplerParameterIiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetSamplerParameterfv=wrapper.wrapper(glGetSamplerParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetSamplerParameterIuiv=wrapper.wrapper(glGetSamplerParameterIuiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetQueryObjecti64v=wrapper.wrapper(glGetQueryObjecti64v).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetQueryObjectui64v=wrapper.wrapper(glGetQueryObjectui64v).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glVertexAttribP1uiv=wrapper.wrapper(glVertexAttribP1uiv).setInputArraySize(
    'value', 1
)
glVertexAttribP2uiv=wrapper.wrapper(glVertexAttribP2uiv).setInputArraySize(
    'value', 1
)
glVertexAttribP3uiv=wrapper.wrapper(glVertexAttribP3uiv).setInputArraySize(
    'value', 1
)
glVertexAttribP4uiv=wrapper.wrapper(glVertexAttribP4uiv).setInputArraySize(
    'value', 1
)
glVertexP2uiv=wrapper.wrapper(glVertexP2uiv).setInputArraySize(
    'value', 1
)
glVertexP3uiv=wrapper.wrapper(glVertexP3uiv).setInputArraySize(
    'value', 1
)
glVertexP4uiv=wrapper.wrapper(glVertexP4uiv).setInputArraySize(
    'value', 1
)
glTexCoordP1uiv=wrapper.wrapper(glTexCoordP1uiv).setInputArraySize(
    'coords', 1
)
glTexCoordP2uiv=wrapper.wrapper(glTexCoordP2uiv).setInputArraySize(
    'coords', 1
)
glTexCoordP3uiv=wrapper.wrapper(glTexCoordP3uiv).setInputArraySize(
    'coords', 1
)
glTexCoordP4uiv=wrapper.wrapper(glTexCoordP4uiv).setInputArraySize(
    'coords', 1
)
glMultiTexCoordP1uiv=wrapper.wrapper(glMultiTexCoordP1uiv).setInputArraySize(
    'coords', 1
)
glMultiTexCoordP2uiv=wrapper.wrapper(glMultiTexCoordP2uiv).setInputArraySize(
    'coords', 1
)
glMultiTexCoordP3uiv=wrapper.wrapper(glMultiTexCoordP3uiv).setInputArraySize(
    'coords', 1
)
glMultiTexCoordP4uiv=wrapper.wrapper(glMultiTexCoordP4uiv).setInputArraySize(
    'coords', 1
)
glNormalP3uiv=wrapper.wrapper(glNormalP3uiv).setInputArraySize(
    'coords', 1
)
glColorP3uiv=wrapper.wrapper(glColorP3uiv).setInputArraySize(
    'color', 1
)
glColorP4uiv=wrapper.wrapper(glColorP4uiv).setInputArraySize(
    'color', 1
)
glSecondaryColorP3uiv=wrapper.wrapper(glSecondaryColorP3uiv).setInputArraySize(
    'color', 1
)
### END AUTOGENERATED SECTION



'''OpenGL extension AMD.shader_image_load_store_lod

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.shader_image_load_store_lod to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension was developed based on the ARB_shader_image_load_store
	extension to allow implementations supporting loads and stores on
	mipmap texture images.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/shader_image_load_store_lod.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.shader_image_load_store_lod import *
from OpenGL.raw.GL.AMD.shader_image_load_store_lod import _EXTENSION_NAME

def glInitShaderImageLoadStoreLodAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension KHR.shader_subgroup

This module customises the behaviour of the 
OpenGL.raw.GL.KHR.shader_subgroup to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables support for the KHR_shader_subgroup shading
	language extension in OpenGL and OpenGL ES.
	
	The extension adds API queries to be able to query
	
	  - the size of subgroups in this implementation (SUBGROUP_SIZE_KHR)
	  - which shader stages support subgroup operations
	    (SUBGROUP_SUPPORTED_STAGES_KHR)
	  - which subgroup features are supported (SUBGROUP_SUPPORTED_FEATURES_KHR)
	  - whether quad subgroup operations are supported in all
	    stages supporting subgroup operations (SUBGROUP_QUAD_ALL_STAGES_KHR)
	
	In OpenGL implementations supporting SPIR-V, this extension enables the
	minimal subset of SPIR-V 1.3 which is required to support the subgroup
	features that are supported by the implementation.
	
	In OpenGL ES implementations, this extension does NOT add support for
	SPIR-V or for any of the built-in shading language functions (8.18)
	that have genDType (double) prototypes.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/KHR/shader_subgroup.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.KHR.shader_subgroup import *
from OpenGL.raw.GL.KHR.shader_subgroup import _EXTENSION_NAME

def glInitShaderSubgroupKHR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension INTEL.framebuffer_CMAA

This module customises the behaviour of the 
OpenGL.raw.GL.INTEL.framebuffer_CMAA to provide a more 
Python-friendly API

Overview (from the spec)
	
	Multisampling is a mechanism to antialias all GL primitives and is part of
	the GL specification.
	
	Better visual quality can be achieved by applying multisampling. However,
	on certain platforms it comes at a high performance cost. In general, the
	greater number of samples per pixel, the bigger the cost.
	
	Conservative Morphological Anti-Aliasing (CMAA) is an alternative approach
	to antialiasing, which operates on the final image. This post processing
	technique results in image quality comparable to multisampling at much
	lower cost and better performance.
	
	This extension incorporates an optimized CMAA algorithm implementation into
	the GL implementation.
	
	For more information on CMAA refer to http://software.intel.com.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/INTEL/framebuffer_CMAA.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.INTEL.framebuffer_CMAA import *
from OpenGL.raw.GL.INTEL.framebuffer_CMAA import _EXTENSION_NAME

def glInitFramebufferCmaaINTEL():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.occlusion_query

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.occlusion_query to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines a mechanism whereby an application can query
	the number of pixels (or, more precisely, samples) drawn by a
	primitive or group of primitives.
	
	The primary purpose of such a query (hereafter referred to as an
	"occlusion query") is to determine the visibility of an object.
	Typically, the application will render the major occluders in the
	scene, then perform an occlusion query for the bounding box of each
	detail object in the scene.  Only if said bounding box is visible,
	i.e., if at least one sample is drawn, should the corresponding object
	be drawn.
	
	The earlier HP_occlusion_test extension defined a similar mechanism,
	but it had two major shortcomings.
	
	- It returned the result as a simple GL_TRUE/GL_FALSE result, when in
	  fact it is often useful to know exactly how many samples were
	  drawn.
	- It provided only a simple "stop-and-wait" model for using multiple
	  queries.  The application begins an occlusion test and ends it;
	  then, at some later point, it asks for the result, at which point
	  the driver must stop and wait until the result from the previous
	  test is back before the application can even begin the next one.
	  This is a very simple model, but its performance is mediocre when
	  an application wishes to perform many queries, and it eliminates
	  most of the opportunities for parallelism between the CPU and GPU.
	
	This extension solves both of those problems.  It returns as its
	result the number of samples that pass the depth and stencil tests,
	and it encapsulates occlusion queries in "query objects" that allow
	applications to issue many queries before asking for the result of
	any one.  As a result, they can overlap the time it takes for the
	occlusion query results to be returned with other, more useful work,
	such as rendering other parts of the scene or performing other
	computations on the CPU.
	
	There are many situations where a pixel/sample count, rather than a
	boolean result, is useful.
	
	- Objects that are visible but cover only a very small number of
	  pixels can be skipped at a minimal reduction of image quality.
	- Knowing exactly how many pixels an object might cover may help the
	  application decide which level-of-detail model should be used.  If
	  only a few pixels are visible, a low-detail model may be
	  acceptable.
	- "Depth peeling" techniques, such as order-independent transparency,
	  need to know when to stop rendering more layers; it is difficult to
	  determine a priori how many layers are needed.  A boolean result
	  allows applications to stop when more layers will not affect the
	  image at all, but this will likely result in unacceptable
	  performance.  Instead, it makes more sense to stop rendering when
	  the number of pixels in each layer falls below a given threshold.
	- Occlusion queries can replace glReadPixels of the depth buffer to
	  determine whether (for example) a light source is visible for the
	  purposes of a lens flare effect or a halo to simulate glare.  Pixel
	  counts allow you to compute the percentage of the light source that
	  is visible, and the brightness of these effects can be modulated
	  accordingly.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/occlusion_query.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.occlusion_query import *
from OpenGL.raw.GL.ARB.occlusion_query import _EXTENSION_NAME

def glInitOcclusionQueryARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGenQueriesARB=wrapper.wrapper(glGenQueriesARB).setOutput(
    'ids',size=lambda x:(x,),pnameArg='n',orPassIn=True
)
# INPUT glDeleteQueriesARB.ids size not checked against n
glDeleteQueriesARB=wrapper.wrapper(glDeleteQueriesARB).setInputArraySize(
    'ids', None
)
glGetQueryivARB=wrapper.wrapper(glGetQueryivARB).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetQueryObjectivARB=wrapper.wrapper(glGetQueryObjectivARB).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetQueryObjectuivARB=wrapper.wrapper(glGetQueryObjectuivARB).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION
from OpenGL.lazywrapper import lazy as _lazy

@_lazy( glDeleteQueriesARB )
def glDeleteQueriesARB( baseOperation, n, ids=None ):
    """Delete the given queries 
    
    n -- either the number of queries to delete, or an array of query values 
    ids -- if provided, the array/pointer to the queries to delete 
    """
    if ids is None:
        ids = arrays.GLuintArray.asArray( n )
        n = arrays.GLuintArray.arraySize( ids )
    else:
        ids = arrays.GLuintArray.asArray( ids )
    return baseOperation( n,ids )



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLSC2 import _types as _cs
# End users want this...
from OpenGL.raw.GLSC2._types import *
from OpenGL.raw.GLSC2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLSC2_OES_rgb8_rgba8'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLSC2,'GLSC2_OES_rgb8_rgba8',error_checker=_errors._error_checker)
GL_RGB8_OES=_C('GL_RGB8_OES',0x8051)
GL_RGBA8_OES=_C('GL_RGBA8_OES',0x8058)




'''OpenGL extension AMD.gpu_shader_int16

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.gpu_shader_int16 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension was developed to allow implementations supporting 16-bit
	integers to expose the feature in GLSL.
	
	The extension introduces the following features for all shader types:
	
	  * new built-in functions to pack and unpack 32-bit integer types into a
	    two-component 16-bit integer vector;
	
	  * new built-in functions to convert half-precision floating-point
	    values to or from their 16-bit integer bit encodings;
	
	  * vector relational functions supporting comparisons of vectors of
	    16-bit integer types; and
	
	  * common functions abs, frexp, ldexp, sign, min, max, clamp, and mix
	    supporting arguments of 16-bit integer types.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/gpu_shader_int16.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.gpu_shader_int16 import *
from OpenGL.raw.GL.AMD.gpu_shader_int16 import _EXTENSION_NAME

def glInitGpuShaderInt16AMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Holds the import-time constants for various configuration flags"""
from OpenGL import (
    ERROR_CHECKING,
    ERROR_LOGGING,
    ERROR_ON_COPY,
    ARRAY_SIZE_CHECKING,
    STORE_POINTERS,
    WARN_ON_FORMAT_UNAVAILABLE,
    FORWARD_COMPATIBLE_ONLY,
    SIZE_1_ARRAY_UNPACK,
    USE_ACCELERATE,
    CONTEXT_CHECKING,

    FULL_LOGGING,
    ALLOW_NUMPY_SCALARS,
    UNSIGNED_BYTE_IMAGES_AS_STRING,
    MODULE_ANNOTATIONS,
    TYPE_ANNOTATIONS,
)



'''OpenGL extension ATI.vertex_streams

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.vertex_streams to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds the ability to handle sets of auxilliary
	vertex and normal coordinates. These sets of auxilliary
	coordinates are termed streams, and can be routed selectively
	into the blend stages provided by the vertex blending extension.
	This functionality enables software animation techniques such
	as keyframe vertex morphing.
	
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/vertex_streams.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ATI.vertex_streams import *
from OpenGL.raw.GL.ATI.vertex_streams import _EXTENSION_NAME

def glInitVertexStreamsATI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glVertexStream1svATI=wrapper.wrapper(glVertexStream1svATI).setInputArraySize(
    'coords', 1
)
glVertexStream1ivATI=wrapper.wrapper(glVertexStream1ivATI).setInputArraySize(
    'coords', 1
)
glVertexStream1fvATI=wrapper.wrapper(glVertexStream1fvATI).setInputArraySize(
    'coords', 1
)
glVertexStream1dvATI=wrapper.wrapper(glVertexStream1dvATI).setInputArraySize(
    'coords', 1
)
glVertexStream2svATI=wrapper.wrapper(glVertexStream2svATI).setInputArraySize(
    'coords', 2
)
glVertexStream2ivATI=wrapper.wrapper(glVertexStream2ivATI).setInputArraySize(
    'coords', 2
)
glVertexStream2fvATI=wrapper.wrapper(glVertexStream2fvATI).setInputArraySize(
    'coords', 2
)
glVertexStream2dvATI=wrapper.wrapper(glVertexStream2dvATI).setInputArraySize(
    'coords', 2
)
glVertexStream3svATI=wrapper.wrapper(glVertexStream3svATI).setInputArraySize(
    'coords', 3
)
glVertexStream3ivATI=wrapper.wrapper(glVertexStream3ivATI).setInputArraySize(
    'coords', 3
)
glVertexStream3fvATI=wrapper.wrapper(glVertexStream3fvATI).setInputArraySize(
    'coords', 3
)
glVertexStream3dvATI=wrapper.wrapper(glVertexStream3dvATI).setInputArraySize(
    'coords', 3
)
glVertexStream4svATI=wrapper.wrapper(glVertexStream4svATI).setInputArraySize(
    'coords', 4
)
glVertexStream4ivATI=wrapper.wrapper(glVertexStream4ivATI).setInputArraySize(
    'coords', 4
)
glVertexStream4fvATI=wrapper.wrapper(glVertexStream4fvATI).setInputArraySize(
    'coords', 4
)
glVertexStream4dvATI=wrapper.wrapper(glVertexStream4dvATI).setInputArraySize(
    'coords', 4
)
glNormalStream3bvATI=wrapper.wrapper(glNormalStream3bvATI).setInputArraySize(
    'coords', 3
)
glNormalStream3svATI=wrapper.wrapper(glNormalStream3svATI).setInputArraySize(
    'coords', 3
)
glNormalStream3ivATI=wrapper.wrapper(glNormalStream3ivATI).setInputArraySize(
    'coords', 3
)
glNormalStream3fvATI=wrapper.wrapper(glNormalStream3fvATI).setInputArraySize(
    'coords', 3
)
glNormalStream3dvATI=wrapper.wrapper(glNormalStream3dvATI).setInputArraySize(
    'coords', 3
)
### END AUTOGENERATED SECTION


'''OpenGL extension IBM.rasterpos_clip

This module customises the behaviour of the 
OpenGL.raw.GL.IBM.rasterpos_clip to provide a more 
Python-friendly API

Overview (from the spec)
	
	IBM_rasterpos_clip extends the semantics of the RasterPos functions.  It
	provides an enable that allows a raster position that would normally be
	clipped to be treated as a valid (albeit out-of-viewport) position.
	
	This extension allows applications to specify geometry-aligned pixel
	primitives that may be partially off-screen.  These primitives are
	tested on a pixel-by-pixel basis without being rejected completely
	because of an invalid raster position.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/IBM/rasterpos_clip.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.IBM.rasterpos_clip import *
from OpenGL.raw.GL.IBM.rasterpos_clip import _EXTENSION_NAME

def glInitRasterposClipIBM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.async_

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.async_ to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a framework for asynchronous OpenGL
	commands.  It also provides commands allowing a program to wait
	for the completion of asynchronous commands.
	
	Asynchronous commands have two properties:
	
	1) Asynchronous commands are non-blocking.  For example, an
	asynchronous ReadPixels command returns control to the program
	immediately rather than blocking until the command completes.
	This property allows the program to issue other OpenGL commands in
	parallel with the execution of commands that normally block.
	
	2) Asynchronous commands may complete out-of-order with respect to
	other OpenGL commands.  For example, an asynchronous TexImage
	command may complete after subsequent OpenGL commands issued by
	the program rather than maintaining the normal serial order of the
	OpenGL command stream.  This property allows the graphics
	accelerator to execute asynchronous commands in parallel with the
	normal command stream, for instance using a secondary path to
	transfer data from or to the host, without doing any dependency
	checking.
	
	Programs that issue asynchronous commands must also be able to
	determine when the commands have completed.  The completion status
	may be needed so that results can be retrieved (e.g. the image
	data from a ReadPixels command) or so that dependent commands can
	be issued (e.g. drawing commands that use texture data downloaded
	by an earlier asynchronous command).  This extension provides
	fine-grain control over asynchronous commands by introducing a
	mechanism for determining the status of individual commands.
	
	Each invocation of an asynchronous command is associated with an
	integer called a "marker."  A program specifies a marker before it
	issues an asynchronous command.  The program may later issue a
	command to query if any asynchronous commands have completed.  The
	query commands return a marker to identify the command that
	completed.  This extension provides both blocking and non-blocking
	query commands.
	
	This extension does not define any asynchronous commands.
	See SGIX_async_pixel for the asynchronous pixel commands.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/async_.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.async_ import *
from OpenGL.raw.GL.SGIX.async_ import _EXTENSION_NAME

def glInitAsyncSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glFinishAsyncSGIX=wrapper.wrapper(glFinishAsyncSGIX).setOutput(
    'markerp',size=(1,),orPassIn=True
)
glPollAsyncSGIX=wrapper.wrapper(glPollAsyncSGIX).setOutput(
    'markerp',size=(1,),orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.occlusion_query_boolean

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.occlusion_query_boolean to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines a mechanism whereby an application can
	query whether any pixels (or, more precisely, samples) are drawn
	by a primitive or group of primitives.
	
	The primary purpose of such a query (hereafter referred to as an
	"occlusion query") is to determine the visibility of an object.
	Typically, the application will render the major occluders in the
	scene, then perform an occlusion query for each detail object in
	the scene. On subsequent frames, the previous results of the
	occlusion queries can be used to decide whether to draw an object
	or not.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/occlusion_query_boolean.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.occlusion_query_boolean import *
from OpenGL.raw.GLES2.EXT.occlusion_query_boolean import _EXTENSION_NAME

def glInitOcclusionQueryBooleanEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGenQueriesEXT.ids size not checked against n
glGenQueriesEXT=wrapper.wrapper(glGenQueriesEXT).setInputArraySize(
    'ids', None
)
# INPUT glDeleteQueriesEXT.ids size not checked against n
glDeleteQueriesEXT=wrapper.wrapper(glDeleteQueriesEXT).setInputArraySize(
    'ids', None
)
# INPUT glGetQueryivEXT.params size not checked against 'pname'
glGetQueryivEXT=wrapper.wrapper(glGetQueryivEXT).setInputArraySize(
    'params', None
)
# INPUT glGetQueryObjectuivEXT.params size not checked against 'pname'
glGetQueryObjectuivEXT=wrapper.wrapper(glGetQueryObjectuivEXT).setInputArraySize(
    'params', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension MESA.framebuffer_flip_y

This module customises the behaviour of the 
OpenGL.raw.GL.MESA.framebuffer_flip_y to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines a new framebuffer parameter,
	GL_FRAMEBUFFER_FLIP_Y_MESA, that changes the behavior of the reads and
	writes to the framebuffer attachment points. When GL_FRAMEBUFFER_FLIP_Y_MESA
	is GL_TRUE, render commands and pixel transfer operations access the
	backing store of each attachment point with an y-inverted coordinate
	system. This y-inversion is relative to the coordinate system set when
	GL_FRAMEBUFFER_FLIP_Y_MESA is GL_FALSE.
	
	Access through TexSubImage2D and similar calls will notice the effect of
	the flip when they are not attached to framebuffer objects because
	GL_FRAMEBUFFER_FLIP_Y_MESA is associated with the framebuffer object and
	not the attachment points.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESA/framebuffer_flip_y.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.MESA.framebuffer_flip_y import *
from OpenGL.raw.GL.MESA.framebuffer_flip_y import _EXTENSION_NAME

def glInitFramebufferFlipYMESA():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGetFramebufferParameterivMESA.params size not checked against 'pname'
glGetFramebufferParameterivMESA=wrapper.wrapper(glGetFramebufferParameterivMESA).setInputArraySize(
    'params', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension OES.surfaceless_context

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.surfaceless_context to provide a more 
Python-friendly API

Overview (from the spec)
	
	Applications that only want to render to framebuffer objects should
	not need to create a throw-away EGL surface (typically a 1x1
	pbuffer) just to get a current context.  The EGL extension
	KHR_surfaceless_context provides a mechanism for making a context
	current without a surface.  This extensions specifies the behaviour
	of OpenGL ES 1.x and OpenGL ES 2.0 when such a context is made
	current.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/surfaceless_context.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.surfaceless_context import *
from OpenGL.raw.GLES1.OES.surfaceless_context import _EXTENSION_NAME

def glInitSurfacelessContextOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.viewport_array

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.viewport_array to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL ES is modeled on a pipeline of operations. The final stage in this
	pipeline before rasterization is the viewport transformation. This stage
	transforms vertices from view space into window coordinates and allows the
	application to specify a rectangular region of screen space into which
	OpenGL ES should draw primitives. Unextended OpenGL ES implementations provide a
	single viewport per context. In order to draw primitives into multiple
	viewports, the OpenGL ES viewport may be changed between several draw calls.
	With the advent of Geometry Shaders, it has become possible for an
	application to amplify geometry and produce multiple output primitives
	for each primitive input to the Geometry Shader. It is possible to direct
	these primitives to render into a selected render target. However, all
	render targets share the same, global OpenGL ES viewport.
	
	This extension enhances OpenGL ES by providing a mechanism to expose multiple
	viewports. Each viewport is specified as a rectangle. The destination
	viewport may be selected per-primitive by the geometry shader. This allows
	the Geometry Shader to produce different versions of primitives destined
	for separate viewport rectangles on the same surface. Additionally, when
	combined with multiple framebuffer attachments, it allows a different
	viewport rectangle to be selected for each. This extension also exposes a
	separate scissor rectangle for each viewport. Finally, the viewport bounds
	are now floating point quantities allowing fractional pixel offsets to be
	applied during the viewport transform.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/viewport_array.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.viewport_array import *
from OpenGL.raw.GLES2.OES.viewport_array import _EXTENSION_NAME

def glInitViewportArrayOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glViewportArrayvOES.v size not checked against 'count'
glViewportArrayvOES=wrapper.wrapper(glViewportArrayvOES).setInputArraySize(
    'v', None
)
glViewportIndexedfvOES=wrapper.wrapper(glViewportIndexedfvOES).setInputArraySize(
    'v', 4
)
# INPUT glScissorArrayvOES.v size not checked against 'count'
glScissorArrayvOES=wrapper.wrapper(glScissorArrayvOES).setInputArraySize(
    'v', None
)
glScissorIndexedvOES=wrapper.wrapper(glScissorIndexedvOES).setInputArraySize(
    'v', 4
)
# INPUT glGetFloati_vOES.data size not checked against 'target'
glGetFloati_vOES=wrapper.wrapper(glGetFloati_vOES).setInputArraySize(
    'data', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension OES.EGL_image

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.EGL_image to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a mechanism for creating texture and
	renderbuffer objects sharing storage with specified EGLImage objects
	(such objects are referred to as "EGLImage targets").
	
	The companion EGL_KHR_image_base and EGL_KHR_image extensions
	provide the definition and rationale for EGLImage objects.
	
	Other EGL extensions, such as EGL_KHR_gl_texture_2D_image,
	EGL_KHR_gl_texture_cubemap_image, EGL_KHR_gl_texture_3D_image,
	EGL_KHR_gl_renderbuffer_image, and EGL_KHR_vg_parent_image, define
	the related functionality of creating EGLImage objects from
	"EGLImage sources" such as OpenGL ES texture or renderbuffers or
	OpenVG VGImage objects.
	
	EGL extension specifications are located in the EGL Registry at
	
	    http://www.khronos.org/registry/egl/

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/EGL_image.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.EGL_image import *
from OpenGL.raw.GLES2.OES.EGL_image import _EXTENSION_NAME

def glInitEglImageOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.separate_shader_objects

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.separate_shader_objects to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/separate_shader_objects.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.separate_shader_objects import *
from OpenGL.raw.GLES2.EXT.separate_shader_objects import _EXTENSION_NAME

def glInitSeparateShaderObjectsEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glCreateShaderProgramvEXT.strings size not checked against count
glCreateShaderProgramvEXT=wrapper.wrapper(glCreateShaderProgramvEXT).setInputArraySize(
    'strings', None
)
# INPUT glDeleteProgramPipelinesEXT.pipelines size not checked against n
glDeleteProgramPipelinesEXT=wrapper.wrapper(glDeleteProgramPipelinesEXT).setInputArraySize(
    'pipelines', None
)
# INPUT glGenProgramPipelinesEXT.pipelines size not checked against n
glGenProgramPipelinesEXT=wrapper.wrapper(glGenProgramPipelinesEXT).setInputArraySize(
    'pipelines', None
)
# INPUT glGetProgramPipelineInfoLogEXT.infoLog size not checked against bufSize
glGetProgramPipelineInfoLogEXT=wrapper.wrapper(glGetProgramPipelineInfoLogEXT).setInputArraySize(
    'infoLog', None
).setInputArraySize(
    'length', 1
)
# INPUT glProgramUniform1fvEXT.value size not checked against count
glProgramUniform1fvEXT=wrapper.wrapper(glProgramUniform1fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform1ivEXT.value size not checked against count
glProgramUniform1ivEXT=wrapper.wrapper(glProgramUniform1ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2fvEXT.value size not checked against count*2
glProgramUniform2fvEXT=wrapper.wrapper(glProgramUniform2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2ivEXT.value size not checked against count*2
glProgramUniform2ivEXT=wrapper.wrapper(glProgramUniform2ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3fvEXT.value size not checked against count*3
glProgramUniform3fvEXT=wrapper.wrapper(glProgramUniform3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3ivEXT.value size not checked against count*3
glProgramUniform3ivEXT=wrapper.wrapper(glProgramUniform3ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4fvEXT.value size not checked against count*4
glProgramUniform4fvEXT=wrapper.wrapper(glProgramUniform4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4ivEXT.value size not checked against count*4
glProgramUniform4ivEXT=wrapper.wrapper(glProgramUniform4ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2fvEXT.value size not checked against count*4
glProgramUniformMatrix2fvEXT=wrapper.wrapper(glProgramUniformMatrix2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3fvEXT.value size not checked against count*9
glProgramUniformMatrix3fvEXT=wrapper.wrapper(glProgramUniformMatrix3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4fvEXT.value size not checked against count*16
glProgramUniformMatrix4fvEXT=wrapper.wrapper(glProgramUniformMatrix4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform1uivEXT.value size not checked against count
glProgramUniform1uivEXT=wrapper.wrapper(glProgramUniform1uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2uivEXT.value size not checked against count*2
glProgramUniform2uivEXT=wrapper.wrapper(glProgramUniform2uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3uivEXT.value size not checked against count*3
glProgramUniform3uivEXT=wrapper.wrapper(glProgramUniform3uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4uivEXT.value size not checked against count*4
glProgramUniform4uivEXT=wrapper.wrapper(glProgramUniform4uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4fvEXT.value size not checked against count*16
glProgramUniformMatrix4fvEXT=wrapper.wrapper(glProgramUniformMatrix4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x3fvEXT.value size not checked against count*6
glProgramUniformMatrix2x3fvEXT=wrapper.wrapper(glProgramUniformMatrix2x3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x2fvEXT.value size not checked against count*6
glProgramUniformMatrix3x2fvEXT=wrapper.wrapper(glProgramUniformMatrix3x2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x4fvEXT.value size not checked against count*8
glProgramUniformMatrix2x4fvEXT=wrapper.wrapper(glProgramUniformMatrix2x4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x2fvEXT.value size not checked against count*8
glProgramUniformMatrix4x2fvEXT=wrapper.wrapper(glProgramUniformMatrix4x2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x4fvEXT.value size not checked against count*12
glProgramUniformMatrix3x4fvEXT=wrapper.wrapper(glProgramUniformMatrix3x4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x3fvEXT.value size not checked against count*12
glProgramUniformMatrix4x3fvEXT=wrapper.wrapper(glProgramUniformMatrix4x3fvEXT).setInputArraySize(
    'value', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.shader_framebuffer_fetch

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.shader_framebuffer_fetch to provide a more 
Python-friendly API

Overview (from the spec)
	
	Conventional OpenGL blending provides a configurable series of operations
	that can be used to combine the output values from a fragment shader with
	the values already in the framebuffer. While these operations are
	suitable for basic image compositing, other compositing operations or
	operations that treat fragment output as something other than a color
	(normals, for instance) may not be expressible without multiple passes or
	render-to-texture operations.
	
	This extension provides a mechanism whereby a fragment shader may read
	existing framebuffer data as input. This can be used to implement
	compositing operations that would have been inconvenient or impossible with
	fixed-function blending. It can also be used to apply a function to the
	framebuffer color, by writing a shader which uses the existing framebuffer
	color as its only input.
	
	This extension provides two alternative name strings:
	
	 - GL_EXT_shader_framebuffer_fetch guarantees full coherency between
	   framebuffer reads and writes.  If this extension string is exposed, the
	   result of reading from the framebuffer from a fragment shader invocation
	   is guaranteed to reflect values written by any previous overlapping
	   samples in API primitive order, unless requested otherwise in the shader
	   source using the noncoherent layout qualifier.
	
	 - GL_EXT_shader_framebuffer_fetch_non_coherent provides limited implicit
	   coherency guarantees.  Instead, the application is expected to call the
	   FramebufferFetchBarrierEXT command for previous framebuffer writes to
	   become visible to subsequent fragment shader invocations.  For this
	   extension to give well-defined results applications may have to split
	   rendering into multiple passes separated with FramebufferFetchBarrierEXT
	   calls.  The functionality provided by this extension is requested in the
	   shader source using the noncoherent layout qualifier.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/shader_framebuffer_fetch.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.shader_framebuffer_fetch import *
from OpenGL.raw.GLES2.EXT.shader_framebuffer_fetch import _EXTENSION_NAME

def glInitShaderFramebufferFetchEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension QCOM.shader_framebuffer_fetch_noncoherent

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.shader_framebuffer_fetch_noncoherent to provide a more 
Python-friendly API

Overview (from the spec)
	
	Existing extensions such as EXT_shader_framebuffer_fetch and 
	ARM_shader_framebuffer_fetch_depth_stencil allow fragment
	shaders to read existing framebuffer color or depth/stencil data as input.
	This extension adds support for reading those same inputs with 
	relaxed coherency requirements.  This mode can avoid expensive
	per-primitive flushes of the pixel pipeline and may offer performance
	improvements in some implementations.
	
	When the relaxed coherency mode is enabled, reads of the framebuffer data
	by the fragment shader will guarantee defined results only if each sample
	is touched no more than once in any single rendering pass.  The command
	FramebufferFetchBarrierQCOM() is provided to indicate a boundary between
	passes.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/shader_framebuffer_fetch_noncoherent.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.shader_framebuffer_fetch_noncoherent import *
from OpenGL.raw.GLES2.QCOM.shader_framebuffer_fetch_noncoherent import _EXTENSION_NAME

def glInitShaderFramebufferFetchNoncoherentQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARM.mali_program_binary

This module customises the behaviour of the 
OpenGL.raw.GLES2.ARM.mali_program_binary to provide a more 
Python-friendly API

Overview (from the spec)
	
	The OES_get_program_binary extension enables applications to retrieve program
	binaries using GetProgramBinaryOES and reload them using ProgramBinaryOES.
	
	The mechanism for retrieval and reloading of program binaries is vendor
	agnostic, but the binary format itself is vendor specific.
	
	This extension adds a token to identify program binaries that are
	compatible with the ARM Mali family of GPUs.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARM/mali_program_binary.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ARM.mali_program_binary import *
from OpenGL.raw.GLES2.ARM.mali_program_binary import _EXTENSION_NAME

def glInitMaliProgramBinaryARM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_internalformat_query2'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_internalformat_query2',error_checker=_errors._error_checker)
GL_AUTO_GENERATE_MIPMAP=_C('GL_AUTO_GENERATE_MIPMAP',0x8295)
GL_CAVEAT_SUPPORT=_C('GL_CAVEAT_SUPPORT',0x82B8)
GL_CLEAR_BUFFER=_C('GL_CLEAR_BUFFER',0x82B4)
GL_COLOR_COMPONENTS=_C('GL_COLOR_COMPONENTS',0x8283)
GL_COLOR_ENCODING=_C('GL_COLOR_ENCODING',0x8296)
GL_COLOR_RENDERABLE=_C('GL_COLOR_RENDERABLE',0x8286)
GL_COMPUTE_TEXTURE=_C('GL_COMPUTE_TEXTURE',0x82A0)
GL_DEPTH_COMPONENTS=_C('GL_DEPTH_COMPONENTS',0x8284)
GL_DEPTH_RENDERABLE=_C('GL_DEPTH_RENDERABLE',0x8287)
GL_FILTER=_C('GL_FILTER',0x829A)
GL_FRAGMENT_TEXTURE=_C('GL_FRAGMENT_TEXTURE',0x829F)
GL_FRAMEBUFFER_BLEND=_C('GL_FRAMEBUFFER_BLEND',0x828B)
GL_FRAMEBUFFER_RENDERABLE=_C('GL_FRAMEBUFFER_RENDERABLE',0x8289)
GL_FRAMEBUFFER_RENDERABLE_LAYERED=_C('GL_FRAMEBUFFER_RENDERABLE_LAYERED',0x828A)
GL_FULL_SUPPORT=_C('GL_FULL_SUPPORT',0x82B7)
GL_GEOMETRY_TEXTURE=_C('GL_GEOMETRY_TEXTURE',0x829E)
GL_GET_TEXTURE_IMAGE_FORMAT=_C('GL_GET_TEXTURE_IMAGE_FORMAT',0x8291)
GL_GET_TEXTURE_IMAGE_TYPE=_C('GL_GET_TEXTURE_IMAGE_TYPE',0x8292)
GL_IMAGE_CLASS_10_10_10_2=_C('GL_IMAGE_CLASS_10_10_10_2',0x82C3)
GL_IMAGE_CLASS_11_11_10=_C('GL_IMAGE_CLASS_11_11_10',0x82C2)
GL_IMAGE_CLASS_1_X_16=_C('GL_IMAGE_CLASS_1_X_16',0x82BE)
GL_IMAGE_CLASS_1_X_32=_C('GL_IMAGE_CLASS_1_X_32',0x82BB)
GL_IMAGE_CLASS_1_X_8=_C('GL_IMAGE_CLASS_1_X_8',0x82C1)
GL_IMAGE_CLASS_2_X_16=_C('GL_IMAGE_CLASS_2_X_16',0x82BD)
GL_IMAGE_CLASS_2_X_32=_C('GL_IMAGE_CLASS_2_X_32',0x82BA)
GL_IMAGE_CLASS_2_X_8=_C('GL_IMAGE_CLASS_2_X_8',0x82C0)
GL_IMAGE_CLASS_4_X_16=_C('GL_IMAGE_CLASS_4_X_16',0x82BC)
GL_IMAGE_CLASS_4_X_32=_C('GL_IMAGE_CLASS_4_X_32',0x82B9)
GL_IMAGE_CLASS_4_X_8=_C('GL_IMAGE_CLASS_4_X_8',0x82BF)
GL_IMAGE_COMPATIBILITY_CLASS=_C('GL_IMAGE_COMPATIBILITY_CLASS',0x82A8)
GL_IMAGE_FORMAT_COMPATIBILITY_TYPE=_C('GL_IMAGE_FORMAT_COMPATIBILITY_TYPE',0x90C7)
GL_IMAGE_PIXEL_FORMAT=_C('GL_IMAGE_PIXEL_FORMAT',0x82A9)
GL_IMAGE_PIXEL_TYPE=_C('GL_IMAGE_PIXEL_TYPE',0x82AA)
GL_IMAGE_TEXEL_SIZE=_C('GL_IMAGE_TEXEL_SIZE',0x82A7)
GL_INTERNALFORMAT_ALPHA_SIZE=_C('GL_INTERNALFORMAT_ALPHA_SIZE',0x8274)
GL_INTERNALFORMAT_ALPHA_TYPE=_C('GL_INTERNALFORMAT_ALPHA_TYPE',0x827B)
GL_INTERNALFORMAT_BLUE_SIZE=_C('GL_INTERNALFORMAT_BLUE_SIZE',0x8273)
GL_INTERNALFORMAT_BLUE_TYPE=_C('GL_INTERNALFORMAT_BLUE_TYPE',0x827A)
GL_INTERNALFORMAT_DEPTH_SIZE=_C('GL_INTERNALFORMAT_DEPTH_SIZE',0x8275)
GL_INTERNALFORMAT_DEPTH_TYPE=_C('GL_INTERNALFORMAT_DEPTH_TYPE',0x827C)
GL_INTERNALFORMAT_GREEN_SIZE=_C('GL_INTERNALFORMAT_GREEN_SIZE',0x8272)
GL_INTERNALFORMAT_GREEN_TYPE=_C('GL_INTERNALFORMAT_GREEN_TYPE',0x8279)
GL_INTERNALFORMAT_PREFERRED=_C('GL_INTERNALFORMAT_PREFERRED',0x8270)
GL_INTERNALFORMAT_RED_SIZE=_C('GL_INTERNALFORMAT_RED_SIZE',0x8271)
GL_INTERNALFORMAT_RED_TYPE=_C('GL_INTERNALFORMAT_RED_TYPE',0x8278)
GL_INTERNALFORMAT_SHARED_SIZE=_C('GL_INTERNALFORMAT_SHARED_SIZE',0x8277)
GL_INTERNALFORMAT_STENCIL_SIZE=_C('GL_INTERNALFORMAT_STENCIL_SIZE',0x8276)
GL_INTERNALFORMAT_STENCIL_TYPE=_C('GL_INTERNALFORMAT_STENCIL_TYPE',0x827D)
GL_INTERNALFORMAT_SUPPORTED=_C('GL_INTERNALFORMAT_SUPPORTED',0x826F)
GL_MANUAL_GENERATE_MIPMAP=_C('GL_MANUAL_GENERATE_MIPMAP',0x8294)
GL_MAX_COMBINED_DIMENSIONS=_C('GL_MAX_COMBINED_DIMENSIONS',0x8282)
GL_MAX_DEPTH=_C('GL_MAX_DEPTH',0x8280)
GL_MAX_HEIGHT=_C('GL_MAX_HEIGHT',0x827F)
GL_MAX_LAYERS=_C('GL_MAX_LAYERS',0x8281)
GL_MAX_WIDTH=_C('GL_MAX_WIDTH',0x827E)
GL_MIPMAP=_C('GL_MIPMAP',0x8293)
GL_NUM_SAMPLE_COUNTS=_C('GL_NUM_SAMPLE_COUNTS',0x9380)
GL_READ_PIXELS=_C('GL_READ_PIXELS',0x828C)
GL_READ_PIXELS_FORMAT=_C('GL_READ_PIXELS_FORMAT',0x828D)
GL_READ_PIXELS_TYPE=_C('GL_READ_PIXELS_TYPE',0x828E)
GL_RENDERBUFFER=_C('GL_RENDERBUFFER',0x8D41)
GL_SAMPLES=_C('GL_SAMPLES',0x80A9)
GL_SHADER_IMAGE_ATOMIC=_C('GL_SHADER_IMAGE_ATOMIC',0x82A6)
GL_SHADER_IMAGE_LOAD=_C('GL_SHADER_IMAGE_LOAD',0x82A4)
GL_SHADER_IMAGE_STORE=_C('GL_SHADER_IMAGE_STORE',0x82A5)
GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_TEST=_C('GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_TEST',0x82AC)
GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_WRITE=_C('GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_WRITE',0x82AE)
GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_TEST=_C('GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_TEST',0x82AD)
GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_WRITE=_C('GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_WRITE',0x82AF)
GL_SRGB_DECODE_ARB=_C('GL_SRGB_DECODE_ARB',0x8299)
GL_SRGB_READ=_C('GL_SRGB_READ',0x8297)
GL_SRGB_WRITE=_C('GL_SRGB_WRITE',0x8298)
GL_STENCIL_COMPONENTS=_C('GL_STENCIL_COMPONENTS',0x8285)
GL_STENCIL_RENDERABLE=_C('GL_STENCIL_RENDERABLE',0x8288)
GL_TESS_CONTROL_TEXTURE=_C('GL_TESS_CONTROL_TEXTURE',0x829C)
GL_TESS_EVALUATION_TEXTURE=_C('GL_TESS_EVALUATION_TEXTURE',0x829D)
GL_TEXTURE_1D=_C('GL_TEXTURE_1D',0x0DE0)
GL_TEXTURE_1D_ARRAY=_C('GL_TEXTURE_1D_ARRAY',0x8C18)
GL_TEXTURE_2D=_C('GL_TEXTURE_2D',0x0DE1)
GL_TEXTURE_2D_ARRAY=_C('GL_TEXTURE_2D_ARRAY',0x8C1A)
GL_TEXTURE_2D_MULTISAMPLE=_C('GL_TEXTURE_2D_MULTISAMPLE',0x9100)
GL_TEXTURE_2D_MULTISAMPLE_ARRAY=_C('GL_TEXTURE_2D_MULTISAMPLE_ARRAY',0x9102)
GL_TEXTURE_3D=_C('GL_TEXTURE_3D',0x806F)
GL_TEXTURE_BUFFER=_C('GL_TEXTURE_BUFFER',0x8C2A)
GL_TEXTURE_COMPRESSED=_C('GL_TEXTURE_COMPRESSED',0x86A1)
GL_TEXTURE_COMPRESSED_BLOCK_HEIGHT=_C('GL_TEXTURE_COMPRESSED_BLOCK_HEIGHT',0x82B2)
GL_TEXTURE_COMPRESSED_BLOCK_SIZE=_C('GL_TEXTURE_COMPRESSED_BLOCK_SIZE',0x82B3)
GL_TEXTURE_COMPRESSED_BLOCK_WIDTH=_C('GL_TEXTURE_COMPRESSED_BLOCK_WIDTH',0x82B1)
GL_TEXTURE_CUBE_MAP=_C('GL_TEXTURE_CUBE_MAP',0x8513)
GL_TEXTURE_CUBE_MAP_ARRAY=_C('GL_TEXTURE_CUBE_MAP_ARRAY',0x9009)
GL_TEXTURE_GATHER=_C('GL_TEXTURE_GATHER',0x82A2)
GL_TEXTURE_GATHER_SHADOW=_C('GL_TEXTURE_GATHER_SHADOW',0x82A3)
GL_TEXTURE_IMAGE_FORMAT=_C('GL_TEXTURE_IMAGE_FORMAT',0x828F)
GL_TEXTURE_IMAGE_TYPE=_C('GL_TEXTURE_IMAGE_TYPE',0x8290)
GL_TEXTURE_RECTANGLE=_C('GL_TEXTURE_RECTANGLE',0x84F5)
GL_TEXTURE_SHADOW=_C('GL_TEXTURE_SHADOW',0x82A1)
GL_TEXTURE_VIEW=_C('GL_TEXTURE_VIEW',0x82B5)
GL_VERTEX_TEXTURE=_C('GL_VERTEX_TEXTURE',0x829B)
GL_VIEW_CLASS_128_BITS=_C('GL_VIEW_CLASS_128_BITS',0x82C4)
GL_VIEW_CLASS_16_BITS=_C('GL_VIEW_CLASS_16_BITS',0x82CA)
GL_VIEW_CLASS_24_BITS=_C('GL_VIEW_CLASS_24_BITS',0x82C9)
GL_VIEW_CLASS_32_BITS=_C('GL_VIEW_CLASS_32_BITS',0x82C8)
GL_VIEW_CLASS_48_BITS=_C('GL_VIEW_CLASS_48_BITS',0x82C7)
GL_VIEW_CLASS_64_BITS=_C('GL_VIEW_CLASS_64_BITS',0x82C6)
GL_VIEW_CLASS_8_BITS=_C('GL_VIEW_CLASS_8_BITS',0x82CB)
GL_VIEW_CLASS_96_BITS=_C('GL_VIEW_CLASS_96_BITS',0x82C5)
GL_VIEW_CLASS_ASTC_10x10_RGBA=_C('GL_VIEW_CLASS_ASTC_10x10_RGBA',0x9393)
GL_VIEW_CLASS_ASTC_10x5_RGBA=_C('GL_VIEW_CLASS_ASTC_10x5_RGBA',0x9390)
GL_VIEW_CLASS_ASTC_10x6_RGBA=_C('GL_VIEW_CLASS_ASTC_10x6_RGBA',0x9391)
GL_VIEW_CLASS_ASTC_10x8_RGBA=_C('GL_VIEW_CLASS_ASTC_10x8_RGBA',0x9392)
GL_VIEW_CLASS_ASTC_12x10_RGBA=_C('GL_VIEW_CLASS_ASTC_12x10_RGBA',0x9394)
GL_VIEW_CLASS_ASTC_12x12_RGBA=_C('GL_VIEW_CLASS_ASTC_12x12_RGBA',0x9395)
GL_VIEW_CLASS_ASTC_4x4_RGBA=_C('GL_VIEW_CLASS_ASTC_4x4_RGBA',0x9388)
GL_VIEW_CLASS_ASTC_5x4_RGBA=_C('GL_VIEW_CLASS_ASTC_5x4_RGBA',0x9389)
GL_VIEW_CLASS_ASTC_5x5_RGBA=_C('GL_VIEW_CLASS_ASTC_5x5_RGBA',0x938A)
GL_VIEW_CLASS_ASTC_6x5_RGBA=_C('GL_VIEW_CLASS_ASTC_6x5_RGBA',0x938B)
GL_VIEW_CLASS_ASTC_6x6_RGBA=_C('GL_VIEW_CLASS_ASTC_6x6_RGBA',0x938C)
GL_VIEW_CLASS_ASTC_8x5_RGBA=_C('GL_VIEW_CLASS_ASTC_8x5_RGBA',0x938D)
GL_VIEW_CLASS_ASTC_8x6_RGBA=_C('GL_VIEW_CLASS_ASTC_8x6_RGBA',0x938E)
GL_VIEW_CLASS_ASTC_8x8_RGBA=_C('GL_VIEW_CLASS_ASTC_8x8_RGBA',0x938F)
GL_VIEW_CLASS_BPTC_FLOAT=_C('GL_VIEW_CLASS_BPTC_FLOAT',0x82D3)
GL_VIEW_CLASS_BPTC_UNORM=_C('GL_VIEW_CLASS_BPTC_UNORM',0x82D2)
GL_VIEW_CLASS_EAC_R11=_C('GL_VIEW_CLASS_EAC_R11',0x9383)
GL_VIEW_CLASS_EAC_RG11=_C('GL_VIEW_CLASS_EAC_RG11',0x9384)
GL_VIEW_CLASS_ETC2_EAC_RGBA=_C('GL_VIEW_CLASS_ETC2_EAC_RGBA',0x9387)
GL_VIEW_CLASS_ETC2_RGB=_C('GL_VIEW_CLASS_ETC2_RGB',0x9385)
GL_VIEW_CLASS_ETC2_RGBA=_C('GL_VIEW_CLASS_ETC2_RGBA',0x9386)
GL_VIEW_CLASS_RGTC1_RED=_C('GL_VIEW_CLASS_RGTC1_RED',0x82D0)
GL_VIEW_CLASS_RGTC2_RG=_C('GL_VIEW_CLASS_RGTC2_RG',0x82D1)
GL_VIEW_CLASS_S3TC_DXT1_RGB=_C('GL_VIEW_CLASS_S3TC_DXT1_RGB',0x82CC)
GL_VIEW_CLASS_S3TC_DXT1_RGBA=_C('GL_VIEW_CLASS_S3TC_DXT1_RGBA',0x82CD)
GL_VIEW_CLASS_S3TC_DXT3_RGBA=_C('GL_VIEW_CLASS_S3TC_DXT3_RGBA',0x82CE)
GL_VIEW_CLASS_S3TC_DXT5_RGBA=_C('GL_VIEW_CLASS_S3TC_DXT5_RGBA',0x82CF)
GL_VIEW_COMPATIBILITY_CLASS=_C('GL_VIEW_COMPATIBILITY_CLASS',0x82B6)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLsizei,arrays.GLint64Array)
def glGetInternalformati64v(target,internalformat,pname,bufSize,params):pass



'''OpenGL extension QCOM.shader_framebuffer_fetch_rate

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.shader_framebuffer_fetch_rate to provide a more 
Python-friendly API

Overview (from the spec)
	
	When certain built-ins (e.g. gl_LastFragData, gl_LastFragStencilARM)
	are referenced in the shader, the shader is required to execute at sample-rate
	if the attachments are multisampled.  In some use-cases executing such shaders
	at fragment-rate is actually the preferred behavior.  When this extension is
	enabled, such GLSL shaders will execute at fragment-rate and the built-in
	will return a per-fragment value.  This avoids the significant performance
	penalty that would otherwise be incurred with sample-rate shading.
	
	The following built-ins are affected when the this extension is enabled:
	
	    gl_LastFragData      (from EXT_shader_framebuffer_fetch)
	    gl_LastFragDepthARM  (from ARM_shader_framebuffer_fetch_depth_stencil)
	
	The following built-ins are disallowed when this extension is enabled:
	
	    gl_SampleID
	    gl_SamplePosition
	    interpolateAtSample()

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/shader_framebuffer_fetch_rate.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.shader_framebuffer_fetch_rate import *
from OpenGL.raw.GLES2.QCOM.shader_framebuffer_fetch_rate import _EXTENSION_NAME

def glInitShaderFramebufferFetchRateQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.packed_float

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.packed_float to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a new 3-component floating-point texture format
	that fits within a single 32-bit word.  This format stores 5 bits
	of biased exponent per component in the same manner as 16-bit
	floating-point formats, but rather than 10 mantissa bits, the red,
	green, and blue components have 6, 6, and 5 bits respectively.
	Each mantissa is assumed to have an implied leading one except in the
	denorm exponent case.  There is no sign bit so only non-negative
	values can be represented.  Positive infinity, positive denorms,
	and positive NaN values are representable.  The value of the fourth
	component returned by a texture fetch is always 1.0.
	
	This extension also provides support for rendering into an unsigned
	floating-point rendering format with the assumption that the texture
	format described above could also be advertised as an unsigned
	floating-point format for rendering.
	
	The extension also provides a pixel external format for specifying
	packed float values directly.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/packed_float.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.packed_float import *
from OpenGL.raw.GL.EXT.packed_float import _EXTENSION_NAME

def glInitPackedFloatEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.polygon_offset_clamp

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.polygon_offset_clamp to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a new parameter to the polygon offset function
	that clamps the calculated offset to a minimum or maximum value.
	The clamping functionality is useful when polygons are nearly
	parallel to the view direction because their high slopes can result
	in arbitrarily large polygon offsets. In the particular case of
	shadow mapping, the lack of clamping can produce the appearance of
	unwanted holes when the shadow casting polygons are offset beyond
	the shadow receiving polygons, and this problem can be alleviated by
	enforcing a maximum offset value.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/polygon_offset_clamp.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.polygon_offset_clamp import *
from OpenGL.raw.GL.ARB.polygon_offset_clamp import _EXTENSION_NAME

def glInitPolygonOffsetClampARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_texture_compression'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_texture_compression',error_checker=_errors._error_checker)
GL_COMPRESSED_ALPHA_ARB=_C('GL_COMPRESSED_ALPHA_ARB',0x84E9)
GL_COMPRESSED_INTENSITY_ARB=_C('GL_COMPRESSED_INTENSITY_ARB',0x84EC)
GL_COMPRESSED_LUMINANCE_ALPHA_ARB=_C('GL_COMPRESSED_LUMINANCE_ALPHA_ARB',0x84EB)
GL_COMPRESSED_LUMINANCE_ARB=_C('GL_COMPRESSED_LUMINANCE_ARB',0x84EA)
GL_COMPRESSED_RGBA_ARB=_C('GL_COMPRESSED_RGBA_ARB',0x84EE)
GL_COMPRESSED_RGB_ARB=_C('GL_COMPRESSED_RGB_ARB',0x84ED)
GL_COMPRESSED_TEXTURE_FORMATS_ARB=_C('GL_COMPRESSED_TEXTURE_FORMATS_ARB',0x86A3)
GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB=_C('GL_NUM_COMPRESSED_TEXTURE_FORMATS_ARB',0x86A2)
GL_TEXTURE_COMPRESSED_ARB=_C('GL_TEXTURE_COMPRESSED_ARB',0x86A1)
GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB=_C('GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB',0x86A0)
GL_TEXTURE_COMPRESSION_HINT_ARB=_C('GL_TEXTURE_COMPRESSION_HINT_ARB',0x84EF)
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexImage1DARB(target,level,internalformat,width,border,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexImage2DARB(target,level,internalformat,width,height,border,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLint,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexImage3DARB(target,level,internalformat,width,height,depth,border,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexSubImage1DARB(target,level,xoffset,width,format,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexSubImage2DARB(target,level,xoffset,yoffset,width,height,format,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glCompressedTexSubImage3DARB(target,level,xoffset,yoffset,zoffset,width,height,depth,format,imageSize,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,ctypes.c_void_p)
def glGetCompressedTexImageARB(target,level,img):pass



'''OpenGL extension EXT.multiview_draw_buffers

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.multiview_draw_buffers to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows selecting among draw buffers as the
	rendering target. This may be among multiple primary buffers
	pertaining to platform-specific stereoscopic or multiview displays
	or among offscreen framebuffer object color attachments.
	
	To remove any artificial limitations imposed on the number of
	possible buffers, draw buffers are identified not as individual
	enums, but as pairs of values consisting of an enum representing
	buffer locations such as COLOR_ATTACHMENT_EXT or MULTIVIEW_EXT,
	and an integer representing an identifying index of buffers of this
	location. These (location, index) pairs are used to specify draw
	buffer targets using a new DrawBuffersIndexedEXT call.
	
	Rendering to buffers of location MULTIVIEW_EXT associated with the
	context allows rendering to multiview buffers created by EGL using
	EGL_EXT_multiview_window for stereoscopic displays.
	
	Rendering to COLOR_ATTACHMENT_EXT buffers allows implementations to
	increase the number of potential color attachments indefinitely to
	renderbuffers and textures.
	
	This extension allows the traditional quad buffer stereoscopic
	rendering method that has proven effective by indicating a left or
	right draw buffer and rendering to each accordingly, but is also
	dynamic enough to handle an arbitrary number of color buffer targets
	all using the same shader. This grants the user maximum flexibility
	as well as a familiar interface.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/multiview_draw_buffers.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.multiview_draw_buffers import *
from OpenGL.raw.GLES2.EXT.multiview_draw_buffers import _EXTENSION_NAME

def glInitMultiviewDrawBuffersEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawBuffersIndexedEXT.indices size not checked against n
# INPUT glDrawBuffersIndexedEXT.location size not checked against n
glDrawBuffersIndexedEXT=wrapper.wrapper(glDrawBuffersIndexedEXT).setInputArraySize(
    'indices', None
).setInputArraySize(
    'location', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.framebuffer_blit

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.framebuffer_blit to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension modifies EXT_framebuffer_object by splitting the
	framebuffer object binding point into separate DRAW and READ
	bindings.  This allows copying directly from one framebuffer to
	another.  In addition, a new high performance blit function is
	added to facilitate these blits and perform some data conversion
	where allowed.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/framebuffer_blit.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.framebuffer_blit import *
from OpenGL.raw.GL.EXT.framebuffer_blit import _EXTENSION_NAME

def glInitFramebufferBlitEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Implementation of OpenGL errors/exceptions

Note that OpenGL-ctypes will also throw standard errors,
such as TypeError or ValueError when appropriate.

ErrorChecker is an _ErrorChecker instance that allows you
to register a new error-checking function for use 
throughout the system.
"""
import logging
_log = logging.getLogger( 'OpenGL.error' )
from OpenGL import platform, _configflags
from ctypes import ArgumentError
__all__ = (
    "Error",'GLError','GLUError','GLUTError',
    'GLerror','GLUerror','GLUTerror','ArgumentError',
)

class Error( Exception ):
    """Base class for all PyOpenGL-specific exception classes"""
class NoContext( Error ):
    """Raised to indicate that there is no currently active context
    
    Technically almost *any* OpenGL call can segfault if there is 
    no active context.  The OpenGL.CHECK_CONTEXT flag, if enabled 
    will cause this error to be raised whenever a GL or GLU call is 
    issued (via PyOpenGL) if there is no currently valid context.
    """
class CopyError( Error ):
    """Raised to indicate that operation requires data-copying
    
    if you set:
        OpenGL.ERROR_ON_COPY = True 
    
    before importing OpenGL.GL, this error will be raised when 
    a passed argument would require a copy to be made.
    """

class NullFunctionError( Error ):
    """Error raised when an undefined function is called"""

class GLError( Error ):
    """OpenGL core error implementation class
    
    Primary purpose of this error class is to allow for 
    annotating an error with more details about the calling 
    environment so that it's easier to debug errors in the
    wrapping process.
    
    Attributes:
    
        err -- the OpenGL error code for the error 
        result -- the OpenGL result code for the operation
        baseOperation -- the "function" being called
        pyArgs -- the translated set of Python arguments
        cArgs -- the Python objects matching 1:1 the C arguments
        cArguments -- ctypes-level arguments to the operation,
            often raw integers for pointers and the like
        description -- OpenGL description of the error (textual)
    """
    def __init__( 
        self, 
        err=None, 
        result=None, 
        cArguments=None, 
        baseOperation=None, 
        pyArgs=None, 
        cArgs=None,
        description=None,
    ):
        """Initialise the GLError, storing metadata for later display"""
        (
            self.err, self.result, self.cArguments, 
            self.baseOperation, self.pyArgs, self.cArgs,
            self.description
        ) = (
            err, result, cArguments,
            baseOperation, pyArgs, cArgs,
            description
        )
    DISPLAY_ORDER = (
        'err', 
        'description',
        'baseOperation',
        'pyArgs', 
        'cArgs',
        'cArguments',
        'result', 
    )
    def __str__( self ):
        """Create a fully formatted representation of the error"""
        args = []
        for property in self.DISPLAY_ORDER:
            value = getattr( self, property, None )
            if value is not None or property=='description':
                formatFunction = 'format_%s'%(property)
                if hasattr( self, formatFunction ):
                    args.append( getattr(self,formatFunction)( property, value ))
                else:
                    args.append( '%s = %s'%(
                        property,
                        self.shortRepr( value ),
                    ))
        return '%s(\n\t%s\n)'%(self.__class__.__name__, ',\n\t'.join(
            [x for x in args if x]
        ))
    def __repr__( self ):
        """Produce a much shorter version of the error as a string"""
        return '%s( %s )'%(
            self.__class__.__name__,
            ", ".join([x for x in [
                'err=%s'%(self.err),
                self.format_description( 'description', self.description ) or '',
                self.format_baseOperation( 'baseOperation', self.baseOperation ) or '',
            ] if x])
        )
    def format_description( self, property, value ):
        """Format description using GLU's gluErrorString"""
        if value is None and self.err is not None:
            try:
                from OpenGL.GLU import gluErrorString
                self.description = value = gluErrorString( self.err )
            except Exception as err:
                return None
        if value is None:
            return None
        return '%s = %s'%(
            property,
            self.shortRepr( value ),
        )
    def shortRepr( self, value, firstLevel=True ):
        """Retrieve short representation of the given value"""
        if isinstance( value, (list,tuple) ) and value and len(repr(value))>=40:
            if isinstance( value, list ):
                template = '[\n\t\t%s\n\t]'
            else:
                template = '(\n\t\t%s,\n\t)'
            return template%( ",\n\t\t".join(
                [
                    self.shortRepr(x,False) for x in value
                ]
            ))
        r = repr( value )
        if len(r) < 120:
            return r
        else:
            return r[:117] + '...'
    def format_baseOperation( self, property, value ):
        """Format a baseOperation reference for display"""
        if hasattr( value, '__name__' ):
            return '%s = %s'%( property, value.__name__ )
        else:
            return '%s = %r'%( property, value )

class GLUError( Error ):
    """GLU error implementation class"""

class GLUTError( Error ):
    """GLUT error implementation class"""
class EGLError( GLError ):
    """EGL error implementation class"""

if _configflags.ERROR_CHECKING:
    from OpenGL import acceleratesupport
    _ErrorChecker = None
    if acceleratesupport.ACCELERATE_AVAILABLE:
        try:
            from OpenGL_accelerate.errorchecker import _ErrorChecker
        except ImportError as err:
            _log.warning( """OpenGL_accelerate seems to be installed, but unable to import error checking entry point!""" )
    if _ErrorChecker is None:
        class _ErrorChecker( object ):
            """Per-API error-checking object
            
            Attributes:
                _registeredChecker -- the checking function enabled when 
                    not doing onBegin/onEnd processing
                _currentChecker -- currently active checking function
            """
            _getErrors = None
            def __init__( self, platform, baseOperation=None, noErrorResult=0, errorClass=GLError ):
                """Initialize from a platform module/reference"""
                self._isValid = platform.CurrentContextIsValid
                self._getErrors = baseOperation
                self._noErrorResult = noErrorResult
                self._errorClass = errorClass
                if self._getErrors:
                    if _configflags.CONTEXT_CHECKING:
                        self._registeredChecker = self.safeGetError 
                    else:
                        self._registeredChecker = self._getErrors
                else:
                    self._registeredChecker = self.nullGetError
                self._currentChecker = self._registeredChecker
            def __bool__( self ):
                """We are "true" if we actually do anything"""
                if self._registeredChecker is self.nullGetError:
                    return False 
                return True
            def safeGetError( self ):
                """Check for error, testing for context before operation"""
                if self._isValid():
                    return self._getErrors()
                return None 
            def nullGetError( self ):
                """Used as error-checker when no error checking should be done"""
                return self._noErrorResult
            def glCheckError( 
                self,
                result,
                baseOperation=None,
                cArguments=None,
                *args
            ):
                """Base GL Error checker compatible with new ctypes errcheck protocol
                
                This function will raise a GLError with just the calling information
                available at the C-calling level, i.e. the error code, cArguments,
                baseOperation and result.  Higher-level code is responsible for any 
                extra annotations.
                
                Note:
                    glCheckError relies on glBegin/glEnd interactions to 
                    prevent glGetError being called during a glBegin/glEnd 
                    sequence.  If you are calling glBegin/glEnd in C you 
                    should call onBegin and onEnd appropriately.
                """
                err = self._currentChecker()
                if err != self._noErrorResult:
                    raise self._errorClass(
                        err,
                        result,
                        cArguments = cArguments,
                        baseOperation = baseOperation,
                    )
                return result
            def onBegin( self ):
                """Called by glBegin to record the fact that glGetError won't work"""
                self._currentChecker = self.nullGetError
            def onEnd( self ):
                """Called by glEnd to record the fact that glGetError will work"""
                self._currentChecker = self._registeredChecker
else:
    _ErrorChecker = None
# Compatibility with PyOpenGL 2.x series
GLUerror = GLUError
GLerror = GLError 
GLUTerror = GLUTError



'''OpenGL extension APPLE.texture_2D_limited_npot

This module customises the behaviour of the 
OpenGL.raw.GLES1.APPLE.texture_2D_limited_npot to provide a more 
Python-friendly API

Overview (from the spec)
	
	Conventional OpenGL ES 1.X texturing is limited to images with
	power-of-two (POT) dimensions.  APPLE_texture_2D_limited_npot extension 
	relaxes these size restrictions for 2D textures.  The restrictions remain
	in place for cube map and 3D textures, if supported.
	
	There is no additional procedural or enumerant API introduced by this
	extension except that an implementation which exports the extension string
	will allow an application to pass in 2D texture dimensions that may or may
	not be a power of two.
	
	In the absence of OES_texture_npot, which lifts these restrictions, neither
	mipmapping nor wrap modes other than CLAMP_TO_EDGE are supported in 
	conjunction with NPOT 2D textures.  A NPOT 2D texture with a wrap mode that
	is not CLAMP_TO_EDGE or a minfilter that is not NEAREST or LINEAR is 
	considered incomplete.  If such a texture is bound to a texture unit, it is 
	as if texture mapping were disabled for that texture unit.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/texture_2D_limited_npot.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.APPLE.texture_2D_limited_npot import *
from OpenGL.raw.GLES1.APPLE.texture_2D_limited_npot import _EXTENSION_NAME

def glInitTexture2DLimitedNpotAPPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.post_depth_coverage

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.post_depth_coverage to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows the fragment shader to control whether values in
	gl_SampleMaskIn[] reflect the coverage after application of the early
	depth and stencil tests.  This feature can be enabled with the following
	layout qualifier in the fragment shader:
	
	    layout(post_depth_coverage) in;
	
	To use this feature, early fragment tests must also be enabled in the
	fragment shader via:
	
	    layout(early_fragment_tests) in;

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/post_depth_coverage.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.post_depth_coverage import *
from OpenGL.raw.GL.EXT.post_depth_coverage import _EXTENSION_NAME

def glInitPostDepthCoverageEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_NV_shading_rate_image'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_NV_shading_rate_image',error_checker=_errors._error_checker)
GL_MAX_COARSE_FRAGMENT_SAMPLES_NV=_C('GL_MAX_COARSE_FRAGMENT_SAMPLES_NV',0x955F)
GL_SHADING_RATE_16_INVOCATIONS_PER_PIXEL_NV=_C('GL_SHADING_RATE_16_INVOCATIONS_PER_PIXEL_NV',0x956F)
GL_SHADING_RATE_1_INVOCATION_PER_1X2_PIXELS_NV=_C('GL_SHADING_RATE_1_INVOCATION_PER_1X2_PIXELS_NV',0x9566)
GL_SHADING_RATE_1_INVOCATION_PER_2X1_PIXELS_NV=_C('GL_SHADING_RATE_1_INVOCATION_PER_2X1_PIXELS_NV',0x9567)
GL_SHADING_RATE_1_INVOCATION_PER_2X2_PIXELS_NV=_C('GL_SHADING_RATE_1_INVOCATION_PER_2X2_PIXELS_NV',0x9568)
GL_SHADING_RATE_1_INVOCATION_PER_2X4_PIXELS_NV=_C('GL_SHADING_RATE_1_INVOCATION_PER_2X4_PIXELS_NV',0x9569)
GL_SHADING_RATE_1_INVOCATION_PER_4X2_PIXELS_NV=_C('GL_SHADING_RATE_1_INVOCATION_PER_4X2_PIXELS_NV',0x956A)
GL_SHADING_RATE_1_INVOCATION_PER_4X4_PIXELS_NV=_C('GL_SHADING_RATE_1_INVOCATION_PER_4X4_PIXELS_NV',0x956B)
GL_SHADING_RATE_1_INVOCATION_PER_PIXEL_NV=_C('GL_SHADING_RATE_1_INVOCATION_PER_PIXEL_NV',0x9565)
GL_SHADING_RATE_2_INVOCATIONS_PER_PIXEL_NV=_C('GL_SHADING_RATE_2_INVOCATIONS_PER_PIXEL_NV',0x956C)
GL_SHADING_RATE_4_INVOCATIONS_PER_PIXEL_NV=_C('GL_SHADING_RATE_4_INVOCATIONS_PER_PIXEL_NV',0x956D)
GL_SHADING_RATE_8_INVOCATIONS_PER_PIXEL_NV=_C('GL_SHADING_RATE_8_INVOCATIONS_PER_PIXEL_NV',0x956E)
GL_SHADING_RATE_IMAGE_BINDING_NV=_C('GL_SHADING_RATE_IMAGE_BINDING_NV',0x955B)
GL_SHADING_RATE_IMAGE_NV=_C('GL_SHADING_RATE_IMAGE_NV',0x9563)
GL_SHADING_RATE_IMAGE_PALETTE_SIZE_NV=_C('GL_SHADING_RATE_IMAGE_PALETTE_SIZE_NV',0x955E)
GL_SHADING_RATE_IMAGE_TEXEL_HEIGHT_NV=_C('GL_SHADING_RATE_IMAGE_TEXEL_HEIGHT_NV',0x955D)
GL_SHADING_RATE_IMAGE_TEXEL_WIDTH_NV=_C('GL_SHADING_RATE_IMAGE_TEXEL_WIDTH_NV',0x955C)
GL_SHADING_RATE_NO_INVOCATIONS_NV=_C('GL_SHADING_RATE_NO_INVOCATIONS_NV',0x9564)
GL_SHADING_RATE_SAMPLE_ORDER_DEFAULT_NV=_C('GL_SHADING_RATE_SAMPLE_ORDER_DEFAULT_NV',0x95AE)
GL_SHADING_RATE_SAMPLE_ORDER_PIXEL_MAJOR_NV=_C('GL_SHADING_RATE_SAMPLE_ORDER_PIXEL_MAJOR_NV',0x95AF)
GL_SHADING_RATE_SAMPLE_ORDER_SAMPLE_MAJOR_NV=_C('GL_SHADING_RATE_SAMPLE_ORDER_SAMPLE_MAJOR_NV',0x95B0)
@_f
@_p.types(None,_cs.GLuint)
def glBindShadingRateImageNV(texture):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,arrays.GLuintArray)
def glGetShadingRateImagePaletteNV(viewport,entry,rate):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,arrays.GLintArray)
def glGetShadingRateSampleLocationivNV(rate,samples,index,location):pass
@_f
@_p.types(None,_cs.GLboolean)
def glShadingRateImageBarrierNV(synchronize):pass
@_f
@_p.types(None,_cs.GLboolean)
def glShadingRateImageBarrierNV(synchronize):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray)
def glShadingRateImagePaletteNV(viewport,first,count,rates):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLintArray)
def glShadingRateSampleOrderCustomNV(rate,samples,locations):pass
@_f
@_p.types(None,_cs.GLenum)
def glShadingRateSampleOrderNV(order):pass



'''OpenGL extension QCOM.driver_control

This module customises the behaviour of the 
OpenGL.raw.GLES1.QCOM.driver_control to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension exposes special control features in a driver to a
	developer. A use of these controls would be to override state or
	implement special modes of operation. One common example might be an
	IFH or infinitely fast hardware mode. In this mode none of draw
	commands would be sent to the GPU so no image would be displayed,
	but all the driver software overhead would still happen thus
	enabling developers to analyze driver overhead separate from GPU
	performance. Some uses of this extension could invalidate future
	rendering and thus should only be used by developers for debugging
	and performance profiling purposes.
	
	The extension is general enough to allow the implementation to
	choose which controls to expose and to provide a textual description
	of those controls to developers.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/driver_control.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.QCOM.driver_control import *
from OpenGL.raw.GLES1.QCOM.driver_control import _EXTENSION_NAME

def glInitDriverControlQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGetDriverControlsQCOM.driverControls size not checked against size
glGetDriverControlsQCOM=wrapper.wrapper(glGetDriverControlsQCOM).setInputArraySize(
    'driverControls', None
)
# INPUT glGetDriverControlStringQCOM.driverControlString size not checked against bufSize
glGetDriverControlStringQCOM=wrapper.wrapper(glGetDriverControlStringQCOM).setInputArraySize(
    'driverControlString', None
)
### END AUTOGENERATED SECTION


from OpenGL.error import _ErrorChecker, EGLError
from OpenGL import platform as _p

class EGLError( EGLError ):
    @property 
    def err(self):
        from OpenGL.EGL import debug
        return debug.eglErrorName(self.__dict__.get('err'))
    @err.setter 
    def err(self, value):
        self.__dict__['err'] = value

if _ErrorChecker:
    _error_checker = _ErrorChecker( 
        _p.PLATFORM, 
        _p.PLATFORM.EGL.eglGetError, 
        0x3000, # EGL_SUCCESS,
        errorClass = EGLError,
    )
else:
    _ErrorChecker = None



"""Lists/tuples as data-format for storage

Note:
    This implementation is *far* less efficient than using Numpy
    to support lists/tuples, as the code here is all available in
    C-level code there.  This implementation is required to allow
    for usage without numpy installed.
"""
REGISTRY_NAME = 'lists'
import ctypes, _ctypes
# Note: these are the same definitions as for GLES, so we are not cross-polluting
from OpenGL.raw.GL import _types 
from OpenGL.arrays import _arrayconstants as GL_1_1
from OpenGL import constant, error
from OpenGL._configflags import ERROR_ON_COPY
from OpenGL.arrays import formathandler
from OpenGL._bytes import bytes,unicode,as_8_bit
HANDLED_TYPES = (list,tuple)
import operator

def err_on_copy( func ):
    """Decorator which raises informative error if we try to copy while ERROR_ON_COPY"""
    if not ERROR_ON_COPY:
        return func 
    else:
        def raiseErrorOnCopy( self,  value, *args, **named ):
            raise error.CopyError(
                """%s passed, cannot copy with ERROR_ON_COPY set, please use an array type which has native data-pointer support (e.g. numpy or ctypes arrays)"""%( value.__class__.__name__, )
            )
        raiseErrorOnCopy.__name__ = getattr(func,'__name__','raiseErrorOnCopy')
        return raiseErrorOnCopy

class ListHandler( formathandler.FormatHandler ):
    """Storage of array data in Python lists/arrays

    This mechanism, unlike multi-dimensional arrays, is not necessarily
    uniform in type or dimension, so we have to do a lot of extra checks
    to make sure that we get a correctly-structured array.  That, as
    well as the need to copy the arrays in Python code, makes this a far
    less efficient implementation than the numpy implementation, which
    does all the same things, but does them all in C code.

    Note: as an *output* format, this format handler produces ctypes
        arrays, not Python lists, this is done for convenience in coding
        the implementation, mostly.
    """
    @err_on_copy
    def from_param( self, instance, typeCode=None ):
        try:
            return ctypes.byref( instance )
        except (TypeError,AttributeError) as err:
            array = self.asArray( instance, typeCode )
            pp = ctypes.c_void_p( ctypes.addressof( array ) )
            pp._temporary_array_ = (array,)
            return pp
    dataPointer = staticmethod( ctypes.addressof )
    HANDLED_TYPES = HANDLED_TYPES 
    isOutput = True
    @err_on_copy
    @classmethod
    def voidDataPointer( cls, value ):
        """Given value in a known data-pointer type, return void_p for pointer"""
        return ctypes.byref( value )
    @classmethod
    def zeros( cls, dims, typeCode ):
        """Return array of zeros in given size"""
        type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
        for dim in dims:
            type *= dim 
        return type() # should expicitly set to 0s
    @classmethod
    def dimsOf( cls, x ):
        """Calculate total dimension-set of the elements in x
        
        This is *extremely* messy, as it has to track nested arrays
        where the arrays could be different sizes on all sorts of 
        levels...
        """
        try:
            dimensions = [ len(x) ]
        except (TypeError,AttributeError,ValueError) as err:
            return []
        else:
            childDimension = None
            for child in x:
                newDimension = cls.dimsOf( child )
                if childDimension is not None:
                    if newDimension != childDimension:
                        raise ValueError( 
                            """Non-uniform array encountered: %s versus %s"""%(
                                newDimension, childDimension,
                            ), x
                        )

    @classmethod
    def arrayToGLType( cls, value ):
        """Given a value, guess OpenGL type of the corresponding pointer"""

        result = ARRAY_TO_GL_TYPE_MAPPING.get( value._type_ )
        if result is not None:
            return result
        raise TypeError(
            """Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
                value._type_, list(ARRAY_TO_GL_TYPE_MAPPING.keys()), value,
            )
        )
    @classmethod
    def arraySize( cls, value, typeCode = None ):
        """Given a data-value, calculate dimensions for the array"""
        dims = 1
        for base in cls.types( value ):
            length = getattr( base, '_length_', None)
            if length is not None:
                dims *= length
        return dims 
    @classmethod
    def types( cls, value ):
        """Produce iterable producing all composite types"""
        dimObject = value
        while dimObject is not None:
            yield dimObject
            dimObject = getattr( dimObject, '_type_', None )
            if isinstance( dimObject, (bytes,unicode)):
                dimObject = None 
    @classmethod
    def dims( cls, value ):
        """Produce iterable of all dimensions"""
        for base in cls.types( value ):
            length = getattr( base, '_length_', None)
            if length is not None:
                yield length
    @err_on_copy
    @classmethod
    def asArray( cls, value, typeCode=None ):
        """Convert given value to a ctypes array value of given typeCode
        
        This does a *lot* of work just to get the data into the correct
        format.  It's not going to be anywhere near as fast as a numpy
        or similar approach!
        """
        if typeCode is None:
            raise NotImplementedError( """Haven't implemented type-inference for lists yet""" )
        arrayType = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
        if isinstance( value, (list,tuple)):
            subItems = [
                cls.asArray( item, typeCode )
                for item in value
            ]
            if subItems:
                for dim in cls.dimensions( subItems[0] )[::-1]:
                    arrayType *= dim
                arrayType *= len( subItems )
                result = arrayType()
                result[:] = subItems
                return result
        else:
            return arrayType( value )
    @err_on_copy
    @classmethod
    def unitSize( cls, value, typeCode=None ):
        """Determine unit size of an array (if possible)"""
        return tuple(cls.dims(value))[-1]
    @err_on_copy
    @classmethod
    def dimensions( cls, value, typeCode=None ):
        """Determine dimensions of the passed array value (if possible)"""
        return tuple( cls.dims(value) )
    @classmethod
    def arrayByteCount( cls, value, typeCode = None ):
        """Given a data-value, calculate number of bytes required to represent"""
        return ctypes.sizeof( value )


ARRAY_TO_GL_TYPE_MAPPING = {
    _types.GLdouble: GL_1_1.GL_DOUBLE,
    _types.GLfloat: GL_1_1.GL_FLOAT,
    _types.GLint: GL_1_1.GL_INT,
    _types.GLuint: GL_1_1.GL_UNSIGNED_INT,
    _types.GLshort: GL_1_1.GL_SHORT,
    _types.GLushort: GL_1_1.GL_UNSIGNED_SHORT,
        
    _types.GLchar: GL_1_1.GL_CHAR,
    _types.GLbyte: GL_1_1.GL_BYTE,
    _types.GLubyte: GL_1_1.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
    GL_1_1.GL_DOUBLE: _types.GLdouble,
    GL_1_1.GL_FLOAT: _types.GLfloat,
    GL_1_1.GL_INT: _types.GLint,
    GL_1_1.GL_UNSIGNED_INT: _types.GLuint,
    GL_1_1.GL_SHORT: _types.GLshort,
    GL_1_1.GL_UNSIGNED_SHORT: _types.GLushort,
        
    GL_1_1.GL_CHAR: _types.GLchar,
    GL_1_1.GL_BYTE: _types.GLbyte,
    GL_1_1.GL_UNSIGNED_BYTE: _types.GLubyte,
    'f': _types.GLfloat,
    'd': _types.GLdouble,
    'i': _types.GLint,
    'I': _types.GLuint,
    'h': _types.GLshort,
    'H': _types.GLushort,
    'b': _types.GLbyte,
    'B': _types.GLubyte,
    's': _types.GLchar,
}



'''OpenGL extension AMD.program_binary_Z400

This module customises the behaviour of the 
OpenGL.raw.GLES2.AMD.program_binary_Z400 to provide a more 
Python-friendly API

Overview (from the spec)
	
	AMD provides an offline shader compiler as part of its suite of SDK tools
	for AMD's Z400 family of embedded graphics accelerator IP.  This extension
	makes available a program binary format, Z400_BINARY_AMD.
	
	The offline shader compiler accepts a pair of OpenGL Shading Language 
	(GLSL) source shaders: one vertex shader and one fragment shader.  It
	outputs a compiled, optimized, and pre-linked program binary which can then
	be loaded into a program objects via the ProgramBinaryOES command.
	
	Applications are recommended to use the OES_get_program_binary extension's
	program binary retrieval mechanism for install-time shader compilation where
	applicable.  That cross-vendor extension provides the performance benefits
	of loading pre-compiled program binaries, while providing the portability of
	deploying GLSL source shaders with the application rather than vendor-
	specific binaries.  The details of this extension are obviated by the use
	of that extension.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/program_binary_Z400.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.AMD.program_binary_Z400 import *
from OpenGL.raw.GLES2.AMD.program_binary_Z400 import _EXTENSION_NAME

def glInitProgramBinaryZ400AMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension MESAX.texture_stack

This module customises the behaviour of the 
OpenGL.raw.GL.MESAX.texture_stack to provide a more 
Python-friendly API

Overview (from the spec)
	
	There are a number of circumstances where an application may wish to
	blend two textures out of a larger set of textures.  Moreover, in some
	cases the selected textures may vary on a per-fragment basis within
	a polygon.  Several examples include:
	
	   1. High dynamic range textures.  The application stores several
	   different "exposures" of an image as different textures.  On a
	   per-fragment basis, the application selects which exposures are
	   used.
	
	   2. A terrain engine where the altitude of a point determines the
	   texture applied to it.  If the transition is from beach sand to
	   grass to rocks to snow, the application will store each texture
	   in a different texture map, and dynamically select which two
	   textures to blend at run-time.
	
	   3. Storing short video clips in textures.  Each depth slice is a
	   single frame of video.
	
	Several solutions to this problem have been proposed, but they either
	involve using a separate texture unit for each texture map or using 3D
	textures without mipmaps.  Both of these options have major drawbacks.
	
	This extension provides a third alternative that eliminates the major
	drawbacks of both previous methods.  A new texture target,
	TEXTURE_2D_STACK, is added that functions identically to TEXTURE_3D in
	all aspects except the sizes of the non-base level images.  In
	traditional 3D texturing, the size of the N+1 LOD is half the size
	of the N LOD in all three dimensions.  For the TEXTURE_2D_STACK target,
	the height and width of the N+1 LOD is halved, but the depth is the
	same for all levels of detail. The texture then becomes a "stack" of
	2D textures.  The per-fragment texel is selected by the R texture
	coordinate.
	
	References:
	
	    http://www.opengl.org/discussion_boards/cgi_directory/ultimatebb.cgi?ubb=get_topic;f=3;t=011557
	    http://www.opengl.org/discussion_boards/cgi_directory/ultimatebb.cgi?ubb=get_topic;f=3;t=000516
	    http://www.opengl.org/discussion_boards/cgi_directory/ultimatebb.cgi?ubb=get_topic;f=3;t=011903
	    http://www.delphi3d.net/articles/viewarticle.php?article=terraintex.htm

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/MESAX/texture_stack.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.MESAX.texture_stack import *
from OpenGL.raw.GL.MESAX.texture_stack import _EXTENSION_NAME

def glInitTextureStackMESAX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_query_resource'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_query_resource',error_checker=_errors._error_checker)
GL_QUERY_RESOURCE_BUFFEROBJECT_NV=_C('GL_QUERY_RESOURCE_BUFFEROBJECT_NV',0x9547)
GL_QUERY_RESOURCE_MEMTYPE_VIDMEM_NV=_C('GL_QUERY_RESOURCE_MEMTYPE_VIDMEM_NV',0x9542)
GL_QUERY_RESOURCE_RENDERBUFFER_NV=_C('GL_QUERY_RESOURCE_RENDERBUFFER_NV',0x9546)
GL_QUERY_RESOURCE_SYS_RESERVED_NV=_C('GL_QUERY_RESOURCE_SYS_RESERVED_NV',0x9544)
GL_QUERY_RESOURCE_TEXTURE_NV=_C('GL_QUERY_RESOURCE_TEXTURE_NV',0x9545)
GL_QUERY_RESOURCE_TYPE_VIDMEM_ALLOC_NV=_C('GL_QUERY_RESOURCE_TYPE_VIDMEM_ALLOC_NV',0x9540)
@_f
@_p.types(_cs.GLint,_cs.GLenum,_cs.GLint,_cs.GLuint,arrays.GLintArray)
def glQueryResourceNV(queryType,tagId,bufSize,buffer):pass



'''OpenGL extension QCOM.extended_get

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.extended_get to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables instrumenting the driver for debugging of OpenGL ES 
	applications.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/extended_get.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.extended_get import *
from OpenGL.raw.GLES2.QCOM.extended_get import _EXTENSION_NAME

def glInitExtendedGetQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glExtGetBuffersQCOM.buffers size not checked against maxBuffers
glExtGetBuffersQCOM=wrapper.wrapper(glExtGetBuffersQCOM).setInputArraySize(
    'buffers', None
).setInputArraySize(
    'numBuffers', 1
)
# INPUT glExtGetRenderbuffersQCOM.renderbuffers size not checked against maxRenderbuffers
glExtGetRenderbuffersQCOM=wrapper.wrapper(glExtGetRenderbuffersQCOM).setInputArraySize(
    'numRenderbuffers', 1
).setInputArraySize(
    'renderbuffers', None
)
# INPUT glExtGetFramebuffersQCOM.framebuffers size not checked against maxFramebuffers
glExtGetFramebuffersQCOM=wrapper.wrapper(glExtGetFramebuffersQCOM).setInputArraySize(
    'framebuffers', None
).setInputArraySize(
    'numFramebuffers', 1
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_clip_cull_distance'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_clip_cull_distance',error_checker=_errors._error_checker)
GL_CLIP_DISTANCE0_EXT=_C('GL_CLIP_DISTANCE0_EXT',0x3000)
GL_CLIP_DISTANCE1_EXT=_C('GL_CLIP_DISTANCE1_EXT',0x3001)
GL_CLIP_DISTANCE2_EXT=_C('GL_CLIP_DISTANCE2_EXT',0x3002)
GL_CLIP_DISTANCE3_EXT=_C('GL_CLIP_DISTANCE3_EXT',0x3003)
GL_CLIP_DISTANCE4_EXT=_C('GL_CLIP_DISTANCE4_EXT',0x3004)
GL_CLIP_DISTANCE5_EXT=_C('GL_CLIP_DISTANCE5_EXT',0x3005)
GL_CLIP_DISTANCE6_EXT=_C('GL_CLIP_DISTANCE6_EXT',0x3006)
GL_CLIP_DISTANCE7_EXT=_C('GL_CLIP_DISTANCE7_EXT',0x3007)
GL_MAX_CLIP_DISTANCES_EXT=_C('GL_MAX_CLIP_DISTANCES_EXT',0x0D32)
GL_MAX_COMBINED_CLIP_AND_CULL_DISTANCES_EXT=_C('GL_MAX_COMBINED_CLIP_AND_CULL_DISTANCES_EXT',0x82FA)
GL_MAX_CULL_DISTANCES_EXT=_C('GL_MAX_CULL_DISTANCES_EXT',0x82F9)

